# Generated by Django 4.2.13 on 2024-06-04 11:12

import cradmin_legacy.apps.cradmin_temporaryfileuploadstore.models
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cradmin_temporaryfileuploadstore', '0004_auto_20151017_1947'),
    ]

    operations = [
        migrations.AlterField(
            model_name='temporaryfile',
            name='file',
            field=models.FileField(storage=cradmin_legacy.apps.cradmin_temporaryfileuploadstore.models.get_temporary_file_storage, upload_to=cradmin_legacy.apps.cradmin_temporaryfileuploadstore.models.temporary_file_upload_to),
        ),
    ]



import re

from django import forms
from django.core import validators
from django.core.validators import EmailValidator
from django.utils.translation import gettext_lazy


class EmailListField(forms.Field):
    """
    A form field that parses and validates a list of email addresses.

    The input can be a list, tuple or string. If it is a string,
    we split the email addresses by whitespace or comma.
    """

    default_error_messages = {
        "invalid": gettext_lazy("Type email-addresses separated by newlines, whitespace or comma.")
    }
    default_help_text = gettext_lazy("Type email-addresses separated by newlines, whitespace or comma.")
    widget = forms.Textarea
    stringsplit_regex = re.compile(r"(?:\s*,\s*)|\s+")
    email_validator = EmailValidator()

    def __init__(self, *args, **kwargs):
        """
        Works just like any other form field.

        Parameters are just like for any other form field, with the
        addition of the ``extra_help_text`` parameter. The ``extra_help_text``
        parameter can be used to append help text to the default help text.
        """
        if "help_text" not in kwargs:
            kwargs["help_text"] = self.default_help_text
        extra_help_text = kwargs.pop("extra_help_text", None)
        if extra_help_text:
            kwargs["help_text"] = "{} {}".format(kwargs["help_text"], extra_help_text)
        super().__init__(*args, **kwargs)

    def string_to_list(self, value):
        value = value.strip()
        if value.strip():
            return self.stringsplit_regex.split(value)
        else:
            return []

    def to_python_list(self, value):
        if value in validators.EMPTY_VALUES:
            return []
        elif isinstance(value, list):
            return value
        elif isinstance(value, tuple):
            return list(value)
        elif isinstance(value, str):
            return self.string_to_list(value)
        else:
            raise TypeError(f"Invalid type for EmailListField: {type(value)}")

    def validate_list_of_emails(self, list_of_emails):
        for email in list_of_emails:
            try:
                self.email_validator(email)
            except forms.ValidationError:
                errormessage = gettext_lazy("Invalid email address: %(email)s") % {"email": email}
                raise forms.ValidationError(errormessage)

    def to_python(self, value):
        list_of_emails = self.to_python_list(value)
        self.validate_list_of_emails(list_of_emails)
        return list_of_emails



from django.test import TestCase

from cradmin_legacy.renderable import AbstractRenderable


class MinimalRenderable(AbstractRenderable):
    template_name = "cradmin_viewhelpers_testapp/listbuilder/minimal-renderable.django.html"


class TestAbstractRenderable(TestCase):
    def test_get_template_name_no_template_name(self):
        with self.assertRaises(NotImplementedError):
            AbstractRenderable().get_template_name()

    def test_get_template_name_has_template_name(self):
        MinimalRenderable().get_template_name()  # NotImplementedError not raised

    def test_get_context_data(self):
        renderable = AbstractRenderable()
        self.assertEqual({"me": renderable}, renderable.get_context_data())

    def test_render(self):
        self.assertEqual("Test", MinimalRenderable().render().strip())



import json
from django import forms
from django.template.loader import render_to_string


class AceMarkdownWidget(forms.widgets.Textarea):
    template_name = "cradmin_legacy/acemarkdown/widget.django.html"
    directiveconfig = {
        # 'showTextarea': False,
        # 'theme': 'tomorrow'
    }

    @property
    def media(self):
        return forms.Media(
            js=[
                "cradmin_legacy/dist/vendor/js/ace-editor/ace.js",
            ]
        )

    def get_context(self, name, value, attrs):
        attrs = attrs.copy()
        attrs["textarea cradmin-legacy-acemarkdown-textarea"] = ""
        if "required" in attrs:
            attrs["required"] = False
        return {
            "directiveconfig": json.dumps(self.directiveconfig),
            "widget": {"name": name, "value": value, "attrs": attrs},
        }



import datetime

import htmls
from django import test
from django.core.files.base import ContentFile
from model_bakery import baker

from cradmin_legacy import automodelform
from cradmin_legacy.apps.cradmin_imagearchive.tests.helpers import create_image
from cradmin_legacy.tests.test_automodelform.cradmin_automodelform_testapp.models import AutoFormTestModel
from cradmin_legacy import datetimeutils
from cradmin_legacy.python2_compatibility import mock


class TestModelForm(test.TestCase):
    def test_datetime_field_without_default_no_instance(self):
        class MyModelForm(automodelform.ModelForm):
            class Meta:
                model = AutoFormTestModel
                fields = ["datetimefield_without_default"]

        selector = htmls.S(MyModelForm().as_ul())
        self.assertEqual("", selector.one('input[type="text"][name="datetimefield_without_default"]').get("value", ""))
        self.assertTrue(selector.exists(".cradmin-legacy-datepicker-triggerbutton"))

    def test_datetime_field_without_default_with_instance(self):
        class MyModelForm(automodelform.ModelForm):
            class Meta:
                model = AutoFormTestModel
                fields = ["datetimefield_without_default"]

        autoformtestmodelobject = baker.make(
            "cradmin_automodelform_testapp.AutoFormTestModel",
            datetimefield_without_default=datetimeutils.default_timezone_datetime(2015, 12, 24, 12, 0, 0),
        )
        selector = htmls.S(MyModelForm(instance=autoformtestmodelobject).as_ul())
        self.assertEqual(
            "2015-12-24 12:00:00", selector.one('input[type="text"][name="datetimefield_without_default"]')["value"]
        )
        self.assertTrue(selector.exists(".cradmin-legacy-datepicker-triggerbutton"))

    def test_datetime_field_with_default_no_instance(self):
        class MyModelForm(automodelform.ModelForm):
            class Meta:
                model = AutoFormTestModel
                fields = ["datetimefield_with_default"]

        selector = htmls.S(MyModelForm().as_ul())
        self.assertNotEqual("", selector.one('input[type="text"][name="datetimefield_with_default"]')["value"])
        self.assertTrue(selector.exists(".cradmin-legacy-datepicker-triggerbutton"))

    def test_datetime_field_with_default_with_instance(self):
        class MyModelForm(automodelform.ModelForm):
            class Meta:
                model = AutoFormTestModel
                fields = ["datetimefield_with_default"]

        autoformtestmodelobject = baker.make(
            "cradmin_automodelform_testapp.AutoFormTestModel",
            datetimefield_with_default=datetimeutils.default_timezone_datetime(2015, 12, 24, 12, 0, 0),
        )
        selector = htmls.S(MyModelForm(instance=autoformtestmodelobject).as_ul())
        self.assertEqual(
            "2015-12-24 12:00:00", selector.one('input[type="text"][name="datetimefield_with_default"]')["value"]
        )
        self.assertTrue(selector.exists(".cradmin-legacy-datepicker-triggerbutton"))

    def test_date_field_without_default_no_instance(self):
        class MyModelForm(automodelform.ModelForm):
            class Meta:
                model = AutoFormTestModel
                fields = ["datefield_without_default"]

        selector = htmls.S(MyModelForm().as_ul())
        self.assertEqual("", selector.one('input[type="text"][name="datefield_without_default"]').get("value", ""))
        self.assertTrue(selector.exists(".cradmin-legacy-datepicker-triggerbutton"))

    def test_date_field_without_default_with_instance(self):
        class MyModelForm(automodelform.ModelForm):
            class Meta:
                model = AutoFormTestModel
                fields = ["datefield_without_default"]

        autoformtestmodelobject = baker.make(
            "cradmin_automodelform_testapp.AutoFormTestModel", datefield_without_default=datetime.date(2015, 12, 24)
        )
        selector = htmls.S(MyModelForm(instance=autoformtestmodelobject).as_ul())
        self.assertEqual("2015-12-24", selector.one('input[type="text"][name="datefield_without_default"]')["value"])
        self.assertTrue(selector.exists(".cradmin-legacy-datepicker-triggerbutton"))

    def test_date_field_with_default_no_instance(self):
        class MyModelForm(automodelform.ModelForm):
            class Meta:
                model = AutoFormTestModel
                fields = ["datefield_with_default"]

        selector = htmls.S(MyModelForm().as_ul())
        self.assertNotEqual("", selector.one('input[type="text"][name="datefield_with_default"]')["value"])
        self.assertTrue(selector.exists(".cradmin-legacy-datepicker-triggerbutton"))

    def test_date_field_with_default_with_instance(self):
        class MyModelForm(automodelform.ModelForm):
            class Meta:
                model = AutoFormTestModel
                fields = ["datefield_with_default"]

        autoformtestmodelobject = baker.make(
            "cradmin_automodelform_testapp.AutoFormTestModel", datefield_with_default=datetime.date(2015, 12, 24)
        )
        selector = htmls.S(MyModelForm(instance=autoformtestmodelobject).as_ul())
        self.assertEqual("2015-12-24", selector.one('input[type="text"][name="datefield_with_default"]')["value"])
        self.assertTrue(selector.exists(".cradmin-legacy-datepicker-triggerbutton"))

    def test_file_field_without_default_no_instance(self):
        class MyModelForm(automodelform.ModelForm):
            class Meta:
                model = AutoFormTestModel
                fields = ["filefield"]

        selector = htmls.S(MyModelForm().as_ul())
        self.assertTrue(selector.exists('input[type="file"][name="filefield"]'))
        self.assertTrue(selector.exists(".cradmin-legacy-filewidget"))

    def test_file_field_without_default_with_instance(self):
        class MyModelForm(automodelform.ModelForm):
            class Meta:
                model = AutoFormTestModel
                fields = ["filefield"]

        autoformtestmodelobject = baker.make("cradmin_automodelform_testapp.AutoFormTestModel", filefield="test.txt")
        selector = htmls.S(MyModelForm(instance=autoformtestmodelobject).as_ul())
        self.assertTrue(selector.exists('input[type="file"][name="filefield"]'))
        self.assertTrue(selector.exists(".cradmin-legacy-filewidget"))

    def test_image_field_without_default_no_instance(self):
        class MyModelForm(automodelform.ModelForm):
            class Meta:
                model = AutoFormTestModel
                fields = ["imagefield"]

        selector = htmls.S(MyModelForm(view=mock.MagicMock()).as_ul())
        self.assertTrue(selector.exists('input[type="file"][name="imagefield"]'))
        self.assertTrue(selector.exists(".cradmin-legacy-imagewidget"))

    def test_image_field_without_default_with_instance(self):
        class MyModelForm(automodelform.ModelForm):
            class Meta:
                model = AutoFormTestModel
                fields = ["imagefield"]

        autoformtestmodelobject = baker.make("cradmin_automodelform_testapp.AutoFormTestModel")
        testimage = create_image(200, 100)
        autoformtestmodelobject.imagefield.save("testimage.png", ContentFile(testimage))

        mockview = mock.MagicMock()
        mockview.request.build_absolute_uri.return_value = "testimage.png"

        with self.settings(CRADMIN_LEGACY_IMAGEUTILS_IMAGETYPE_MAP={}):
            selector = htmls.S(MyModelForm(view=mockview, instance=autoformtestmodelobject).as_ul())
        self.assertTrue(selector.exists('input[type="file"][name="imagefield"]'))
        self.assertTrue(selector.exists(".cradmin-legacy-imagewidget"))



from . import mixins  # noqa
from . import crudbase  # noqa
from . import create  # noqa
from . import update  # noqa
from . import delete  # noqa
from . import detail  # noqa
from . import formbase  # noqa
from . import listbuilderview  # noqa
from . import multiselect  # noqa
from . import objecttable  # noqa

__all__ = [
    "mixins",
    "crudbase",
    "create",
    "update",
    "delete",
    "detail",
    "formbase",
    "listbuilderview",
    "multiselect",
    "objecttable",
]



from django.core import mail

from django.test import TestCase
from cradmin_legacy.python2_compatibility import mock

from cradmin_legacy.apps.cradmin_invite.invite_url import InviteUrl
from cradmin_legacy.tests.helpers import create_user


class InviteUrlMock(InviteUrl):
    def get_appname(self):
        return "testapp"

    def get_confirm_invite_url(self, token):
        return f"/invite/accept/{token.token}"


class InviteUrlWithStaticTokenMock(InviteUrlMock):
    def generate_generictoken(self, email=None):
        testtoken = mock.MagicMock()
        testtoken.token = "test-token"
        return testtoken


class InviteUrlWithTokenIteratorMock(InviteUrlMock):
    def __init__(self, *args, **kwargs):
        self.tokens = iter(["token1", "token2", "token3"])
        super().__init__(*args, **kwargs)

    def _generate_generictoken(self, email=None):
        testtoken = mock.MagicMock()
        testtoken.token = next(self.tokens)
        return testtoken


class TestSendActivationEmail(TestCase):
    def setUp(self):
        self.testuser = create_user("testuser", email="testuser@example.com")

        # Any object will work as the target of invites.
        # We use a user object since we have that easily available
        self.invite_target = create_user("invitetarget")

    def test_generate_generictoken(self):
        token = InviteUrlMock(
            request=mock.MagicMock(), private=True, content_object=self.invite_target
        ).generate_generictoken()
        self.assertEqual(token.content_object, self.invite_target)

    def test_generate_generictoken_with_email(self):
        token = InviteUrlMock(
            request=mock.MagicMock(), private=True, content_object=self.invite_target
        ).generate_generictoken(email="test@example.com")
        self.assertEqual(token.metadata["email"], "test@example.com")

    def test_generate_generictoken_with_email_and_metadata(self):
        token = InviteUrlMock(
            request=mock.MagicMock(), private=True, content_object=self.invite_target, metadata={"test": 10}
        ).generate_generictoken(email="test@example.com")
        self.assertEqual(token.metadata["email"], "test@example.com")
        self.assertEqual(token.metadata["test"], 10)

    def test_generate_generictoken_with_email_and_metadata_with_email(self):
        token = InviteUrlMock(
            request=mock.MagicMock(),
            private=True,
            content_object=self.invite_target,
            metadata={"email": "existing@example.com"},
        ).generate_generictoken(email="test@example.com")
        self.assertEqual(token.metadata["email"], "existing@example.com")

    def test_send_email(self):
        testrequest = mock.MagicMock()
        testrequest.build_absolute_uri = lambda path: f"http://testserver{path}"

        with self.settings(CRADMIN_LEGACY_SITENAME="Testsite"):
            InviteUrlWithStaticTokenMock(
                request=testrequest, content_object=self.invite_target, private=True
            ).send_email("test@example.com")
        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].subject, "Invitation to Testsite")
        self.assertIn("http://testserver/invite/accept/test-token", mail.outbox[0].alternatives[0][0])
        self.assertIn("Click the button below to accept the invite.", mail.outbox[0].alternatives[0][0])

    def test_send_email_private(self):
        testrequest = mock.MagicMock()
        testrequest.build_absolute_uri = lambda path: f"http://testserver{path}"

        with self.settings(CRADMIN_LEGACY_SITENAME="Testsite"):
            inviteurl = InviteUrlWithTokenIteratorMock(
                request=testrequest, content_object=self.invite_target, private=True
            )
            inviteurl.send_email("test1@example.com", "test2@example.com", "test3@example.com")
        self.assertIn("token1", mail.outbox[0].body)
        self.assertIn("token2", mail.outbox[1].body)
        self.assertIn("token3", mail.outbox[2].body)

    def test_send_email_public(self):
        testrequest = mock.MagicMock()
        testrequest.build_absolute_uri = lambda path: f"http://testserver{path}"

        with self.settings(CRADMIN_LEGACY_SITENAME="Testsite"):
            inviteurl = InviteUrlWithTokenIteratorMock(
                request=testrequest, content_object=self.invite_target, private=False
            )
            inviteurl.send_email("test1@example.com", "test2@example.com", "test3@example.com")
        self.assertIn("token1", mail.outbox[0].body)
        self.assertIn("token1", mail.outbox[1].body)
        self.assertIn("token1", mail.outbox[2].body)

    def test_get_share_url(self):
        testrequest = mock.MagicMock()
        testrequest.build_absolute_uri = lambda path: f"http://testserver{path}"
        self.assertEqual(
            InviteUrlWithStaticTokenMock(
                request=testrequest, private=True, content_object=self.invite_target
            ).get_share_url(),
            "http://testserver/invite/accept/test-token",
        )

    def test_get_share_url_private(self):
        testrequest = mock.MagicMock()
        testrequest.build_absolute_uri = lambda path: f"http://testserver{path}"
        inviteurl = InviteUrlWithTokenIteratorMock(request=testrequest, private=True, content_object=self.invite_target)
        self.assertEqual(inviteurl.get_share_url(), "http://testserver/invite/accept/token1")
        self.assertEqual(inviteurl.get_share_url(), "http://testserver/invite/accept/token2")
        self.assertEqual(inviteurl.get_share_url(), "http://testserver/invite/accept/token3")

    def test_get_share_url_public(self):
        testrequest = mock.MagicMock()
        testrequest.build_absolute_uri = lambda path: f"http://testserver{path}"
        inviteurl = InviteUrlWithTokenIteratorMock(
            request=testrequest, private=False, content_object=self.invite_target
        )
        self.assertEqual(inviteurl.get_share_url(), "http://testserver/invite/accept/token1")
        self.assertEqual(inviteurl.get_share_url(), "http://testserver/invite/accept/token1")
        self.assertEqual(inviteurl.get_share_url(), "http://testserver/invite/accept/token1")



import urllib.parse

from django.forms import widgets
from django.template.loader import render_to_string
from django.utils.translation import gettext_lazy


class ModelChoiceWidget(widgets.TextInput):
    """
    Model choice widget that uses an iframe popup to enable users
    to select their foreign key value.
    """

    #: The template used to render the widget.
    template_name = "cradmin_legacy/widgets/modelchoice.django.html"

    #: Do not override this (if you set this to hidden, the widget is not rendered correctly).
    input_type = "text"

    #: Set this to ``True`` to debug the value of the input field.
    #: Setting this to ``True``, makes the ``type`` of the actually
    #: submitted input field to ``text`` instead of ``hidden``.
    #: Probably only useful for debugging.
    input_field_visible = False

    #: The default select-button text. You can override this in a subclass,
    #: or use the ``selectbutton_text``-argument for the constructor to
    #: change the button text.
    default_selectbutton_text = gettext_lazy("Select ...")

    def __init__(self, queryset, selectview_url, preview="", selectbutton_text=None):
        self.queryset = queryset
        self.preview = preview
        self.selectview_url = selectview_url
        self.selectbutton_text = selectbutton_text or self.default_selectbutton_text
        super().__init__()

    def __make_selectview_url(self, fieldid, current_value):
        return "{}?{}".format(
            self.selectview_url,
            urllib.parse.urlencode(
                {
                    "foreignkey_select_current_value": current_value,
                    "foreignkey_select_fieldid": fieldid,
                }
            ),
        )

    def get_rendered_input_type(self):
        if self.input_field_visible:
            return "text"
        else:
            return "hidden"

    def render(self, name, value, attrs=None, renderer=None):
        if value is None:
            value = ""
        fieldid = attrs["id"]
        return render_to_string(
            self.template_name,
            {
                "preview": self.preview,
                "fieldname": name,
                "fieldid": fieldid,
                "fieldvalue": value,
                "selectview_url": self.__make_selectview_url(fieldid, value),
                "selectbutton_text": self.selectbutton_text,
                "input_type": self.input_type,
                "rendered_input_type": self.get_rendered_input_type(),
            },
        )



from django.test import TestCase
from model_bakery import baker

from cradmin_legacy import datetimeutils
from cradmin_legacy.python2_compatibility import mock
from cradmin_legacy.tests.viewhelpers.cradmin_viewhelpers_testapp.models import FilterTestModel
from cradmin_legacy.viewhelpers import listfilter


class TestBoolean(TestCase):
    def test_no_value(self):
        testfilter = listfilter.django.single.select.Boolean(slug="mycharfield")
        testfilter.set_values(values=[])
        withvalue = baker.make("cradmin_viewhelpers_testapp.FilterTestModel", mycharfield="A testvalue")
        emptyvalue = baker.make("cradmin_viewhelpers_testapp.FilterTestModel", mycharfield="")
        nullvalue = baker.make("cradmin_viewhelpers_testapp.FilterTestModel", mycharfield=None)
        self.assertEqual(
            {withvalue, emptyvalue, nullvalue}, set(testfilter.filter(queryobject=FilterTestModel.objects.all()))
        )

    def test_invalid_value(self):
        testfilter = listfilter.django.single.select.Boolean(slug="mycharfield")
        testfilter.set_values(values=["invalidstuff"])
        withvalue = baker.make("cradmin_viewhelpers_testapp.FilterTestModel", mycharfield="A testvalue")
        emptyvalue = baker.make("cradmin_viewhelpers_testapp.FilterTestModel", mycharfield="")
        nullvalue = baker.make("cradmin_viewhelpers_testapp.FilterTestModel", mycharfield=None)
        self.assertEqual(
            {withvalue, emptyvalue, nullvalue}, set(testfilter.filter(queryobject=FilterTestModel.objects.all()))
        )

    def test_false_booleanfield(self):
        testfilter = listfilter.django.single.select.Boolean(slug="mybooleanfield")
        testfilter.set_values(values=["false"])
        baker.make("cradmin_viewhelpers_testapp.FilterTestModel", mybooleanfield=True)
        falsevalue = baker.make("cradmin_viewhelpers_testapp.FilterTestModel", mybooleanfield=False)
        self.assertEqual({falsevalue}, set(testfilter.filter(queryobject=FilterTestModel.objects.all())))

    def test_true_booleanfield(self):
        testfilter = listfilter.django.single.select.Boolean(slug="mybooleanfield")
        testfilter.set_values(values=["true"])
        truevalue = baker.make("cradmin_viewhelpers_testapp.FilterTestModel", mybooleanfield=True)
        baker.make("cradmin_viewhelpers_testapp.FilterTestModel", mybooleanfield=False)
        self.assertEqual({truevalue}, set(testfilter.filter(queryobject=FilterTestModel.objects.all())))


class TestIsNotNull(TestCase):
    def test_false(self):
        testfilter = listfilter.django.single.select.IsNotNull(slug="myintnullfield")
        testfilter.set_values(values=["false"])
        baker.make("cradmin_viewhelpers_testapp.FilterTestModel", myintnullfield=10)
        nullvalue = baker.make("cradmin_viewhelpers_testapp.FilterTestModel", myintnullfield=None)
        self.assertEqual({nullvalue}, set(testfilter.filter(queryobject=FilterTestModel.objects.all())))

    def test_true(self):
        testfilter = listfilter.django.single.select.IsNotNull(slug="myintnullfield")
        testfilter.set_values(values=["true"])
        withvalue = baker.make("cradmin_viewhelpers_testapp.FilterTestModel", myintnullfield=10)
        baker.make("cradmin_viewhelpers_testapp.FilterTestModel", myintnullfield=None)
        self.assertEqual({withvalue}, set(testfilter.filter(queryobject=FilterTestModel.objects.all())))

    def test_true_zero(self):
        testfilter = listfilter.django.single.select.IsNotNull(slug="myintnullfield")
        testfilter.set_values(values=["true"])
        withvalue = baker.make("cradmin_viewhelpers_testapp.FilterTestModel", myintnullfield=0)
        baker.make("cradmin_viewhelpers_testapp.FilterTestModel", myintnullfield=None)
        self.assertEqual({withvalue}, set(testfilter.filter(queryobject=FilterTestModel.objects.all())))


class TestDateTime(TestCase):
    def test_no_value(self):
        testfilter = listfilter.django.single.select.DateTime(slug="mydatetimefield")
        testfilter.set_values(values=[])
        testitem = baker.make("cradmin_viewhelpers_testapp.FilterTestModel")
        self.assertEqual({testitem}, set(testfilter.filter(queryobject=FilterTestModel.objects.all())))

    def test_invalid_value(self):
        testfilter = listfilter.django.single.select.DateTime(slug="mydatetimefield")
        testfilter.set_values(values=["invalidstuff"])
        testitem = baker.make("cradmin_viewhelpers_testapp.FilterTestModel")
        self.assertEqual({testitem}, set(testfilter.filter(queryobject=FilterTestModel.objects.all())))

    def test_today_nomatch(self):
        testfilter = listfilter.django.single.select.DateTime(slug="mydatetimefield")
        testfilter.set_values(values=["today"])
        baker.make(
            "cradmin_viewhelpers_testapp.FilterTestModel",
            mydatetimefield=datetimeutils.default_timezone_datetime(2015, 1, 1),
        )
        with mock.patch(
            "cradmin_legacy.viewhelpers.listfilter.basefilters.single.abstractselect.timezone.now",
            lambda: datetimeutils.default_timezone_datetime(2015, 1, 2),
        ):
            self.assertFalse(testfilter.filter(queryobject=FilterTestModel.objects.all()).exists())

    def test_today_match(self):
        testfilter = listfilter.django.single.select.DateTime(slug="mydatetimefield")
        testfilter.set_values(values=["today"])
        testitem = baker.make(
            "cradmin_viewhelpers_testapp.FilterTestModel",
            mydatetimefield=datetimeutils.default_timezone_datetime(2015, 1, 1),
        )
        with mock.patch(
            "cradmin_legacy.viewhelpers.listfilter.basefilters.single.abstractselect.timezone.now",
            lambda: datetimeutils.default_timezone_datetime(2015, 1, 1, 12, 30),
        ):
            self.assertEqual({testitem}, set(testfilter.filter(queryobject=FilterTestModel.objects.all())))

    def test_yesterday_nomatch(self):
        testfilter = listfilter.django.single.select.DateTime(slug="mydatetimefield")
        testfilter.set_values(values=["yesterday"])
        baker.make(
            "cradmin_viewhelpers_testapp.FilterTestModel",
            mydatetimefield=datetimeutils.default_timezone_datetime(2015, 1, 10),
        )
        with mock.patch(
            "cradmin_legacy.viewhelpers.listfilter.basefilters.single.abstractselect.timezone.now",
            lambda: datetimeutils.default_timezone_datetime(2015, 1, 12),
        ):
            self.assertFalse(testfilter.filter(queryobject=FilterTestModel.objects.all()).exists())

    def test_yesterday_match(self):
        testfilter = listfilter.django.single.select.DateTime(slug="mydatetimefield")
        testfilter.set_values(values=["yesterday"])
        testitem = baker.make(
            "cradmin_viewhelpers_testapp.FilterTestModel",
            mydatetimefield=datetimeutils.default_timezone_datetime(2015, 1, 10),
        )
        with mock.patch(
            "cradmin_legacy.viewhelpers.listfilter.basefilters.single.abstractselect.timezone.now",
            lambda: datetimeutils.default_timezone_datetime(2015, 1, 11, 12, 30),
        ):
            self.assertEqual({testitem}, set(testfilter.filter(queryobject=FilterTestModel.objects.all())))

    def test_last_seven_days_nomatch(self):
        testfilter = listfilter.django.single.select.DateTime(slug="mydatetimefield")
        testfilter.set_values(values=["last_seven_days"])
        baker.make(
            "cradmin_viewhelpers_testapp.FilterTestModel",
            mydatetimefield=datetimeutils.default_timezone_datetime(2015, 1, 2),
        )
        with mock.patch(
            "cradmin_legacy.viewhelpers.listfilter.basefilters.single.abstractselect.timezone.now",
            lambda: datetimeutils.default_timezone_datetime(2015, 1, 10, 12, 30),
        ):
            self.assertFalse(testfilter.filter(queryobject=FilterTestModel.objects.all()).exists())

    def test_last_seven_days_match(self):
        testfilter = listfilter.django.single.select.DateTime(slug="mydatetimefield")
        testfilter.set_values(values=["last_seven_days"])
        testitem = baker.make(
            "cradmin_viewhelpers_testapp.FilterTestModel",
            mydatetimefield=datetimeutils.default_timezone_datetime(2015, 1, 3),
        )
        with mock.patch(
            "cradmin_legacy.viewhelpers.listfilter.basefilters.single.abstractselect.timezone.now",
            lambda: datetimeutils.default_timezone_datetime(2015, 1, 10, 12, 30),
        ):
            self.assertEqual({testitem}, set(testfilter.filter(queryobject=FilterTestModel.objects.all())))

    def test_this_week_nomatch(self):
        testfilter = listfilter.django.single.select.DateTime(slug="mydatetimefield")
        testfilter.set_values(values=["this_week"])
        baker.make(
            "cradmin_viewhelpers_testapp.FilterTestModel",
            mydatetimefield=datetimeutils.default_timezone_datetime(2015, 11, 29),
        )
        baker.make(
            "cradmin_viewhelpers_testapp.FilterTestModel",
            mydatetimefield=datetimeutils.default_timezone_datetime(2015, 12, 7),
        )
        with mock.patch(
            "cradmin_legacy.viewhelpers.listfilter.basefilters.single.abstractselect.timezone.now",
            lambda: datetimeutils.default_timezone_datetime(2015, 12, 1, 12, 30),
        ):
            self.assertFalse(testfilter.filter(queryobject=FilterTestModel.objects.all()).exists())

    def test_this_week_match(self):
        testfilter = listfilter.django.single.select.DateTime(slug="mydatetimefield")
        testfilter.set_values(values=["this_week"])
        start_of_week = baker.make(
            "cradmin_viewhelpers_testapp.FilterTestModel",
            mydatetimefield=datetimeutils.default_timezone_datetime(2015, 11, 30),
        )
        end_of_week = baker.make(
            "cradmin_viewhelpers_testapp.FilterTestModel",
            mydatetimefield=datetimeutils.default_timezone_datetime(2015, 12, 6, 23),
        )
        with mock.patch(
            "cradmin_legacy.viewhelpers.listfilter.basefilters.single.abstractselect.timezone.now",
            lambda: datetimeutils.default_timezone_datetime(2015, 12, 1, 12, 30),
        ):
            self.assertEqual(
                {start_of_week, end_of_week}, set(testfilter.filter(queryobject=FilterTestModel.objects.all()))
            )

    def test_this_month_nomatch(self):
        testfilter = listfilter.django.single.select.DateTime(slug="mydatetimefield")
        testfilter.set_values(values=["this_month"])
        baker.make(
            "cradmin_viewhelpers_testapp.FilterTestModel",
            mydatetimefield=datetimeutils.default_timezone_datetime(2015, 11, 29),
        )
        baker.make(
            "cradmin_viewhelpers_testapp.FilterTestModel",
            mydatetimefield=datetimeutils.default_timezone_datetime(2016, 1, 1),
        )
        with mock.patch(
            "cradmin_legacy.viewhelpers.listfilter.basefilters.single.abstractselect.timezone.now",
            lambda: datetimeutils.default_timezone_datetime(2015, 12, 7, 12, 30),
        ):
            self.assertFalse(testfilter.filter(queryobject=FilterTestModel.objects.all()).exists())

    def test_this_month_match(self):
        testfilter = listfilter.django.single.select.DateTime(slug="mydatetimefield")
        testfilter.set_values(values=["this_month"])
        start_of_month = baker.make(
            "cradmin_viewhelpers_testapp.FilterTestModel",
            mydatetimefield=datetimeutils.default_timezone_datetime(2015, 12, 1),
        )
        middle_of_month = baker.make(
            "cradmin_viewhelpers_testapp.FilterTestModel",
            mydatetimefield=datetimeutils.default_timezone_datetime(2015, 12, 24),
        )
        end_of_month = baker.make(
            "cradmin_viewhelpers_testapp.FilterTestModel",
            mydatetimefield=datetimeutils.default_timezone_datetime(2015, 12, 31, 23),
        )
        with mock.patch(
            "cradmin_legacy.viewhelpers.listfilter.basefilters.single.abstractselect.timezone.now",
            lambda: datetimeutils.default_timezone_datetime(2015, 12, 7, 12, 30),
        ):
            self.assertEqual(
                {start_of_month, middle_of_month, end_of_month},
                set(testfilter.filter(queryobject=FilterTestModel.objects.all())),
            )

    def test_this_year_nomatch(self):
        testfilter = listfilter.django.single.select.DateTime(slug="mydatetimefield")
        testfilter.set_values(values=["this_year"])
        baker.make(
            "cradmin_viewhelpers_testapp.FilterTestModel",
            mydatetimefield=datetimeutils.default_timezone_datetime(2014, 12, 31),
        )
        baker.make(
            "cradmin_viewhelpers_testapp.FilterTestModel",
            mydatetimefield=datetimeutils.default_timezone_datetime(2016, 1, 1),
        )
        with mock.patch(
            "cradmin_legacy.viewhelpers.listfilter.basefilters.single.abstractselect.timezone.now",
            lambda: datetimeutils.default_timezone_datetime(2015, 12, 24),
        ):
            self.assertFalse(testfilter.filter(queryobject=FilterTestModel.objects.all()).exists())

    def test_this_year_match(self):
        testfilter = listfilter.django.single.select.DateTime(slug="mydatetimefield")
        testfilter.set_values(values=["this_year"])
        start_of_year = baker.make(
            "cradmin_viewhelpers_testapp.FilterTestModel",
            mydatetimefield=datetimeutils.default_timezone_datetime(2015, 1, 1),
        )
        middle_of_year = baker.make(
            "cradmin_viewhelpers_testapp.FilterTestModel",
            mydatetimefield=datetimeutils.default_timezone_datetime(2015, 6, 1),
        )
        end_of_year = baker.make(
            "cradmin_viewhelpers_testapp.FilterTestModel",
            mydatetimefield=datetimeutils.default_timezone_datetime(2015, 12, 31, 23),
        )
        with mock.patch(
            "cradmin_legacy.viewhelpers.listfilter.basefilters.single.abstractselect.timezone.now",
            lambda: datetimeutils.default_timezone_datetime(2015, 12, 24),
        ):
            self.assertEqual(
                {start_of_year, middle_of_year, end_of_year},
                set(testfilter.filter(queryobject=FilterTestModel.objects.all())),
            )


class TestNullDateTime(TestCase):
    def test_is_null(self):
        testfilter = listfilter.django.single.select.NullDateTime(slug="mynulldatetimefield")
        testfilter.set_values(values=["is_null"])
        baker.make(
            "cradmin_viewhelpers_testapp.FilterTestModel",
            mynulldatetimefield=datetimeutils.default_timezone_datetime(2014, 12, 31),
        )
        isnull = baker.make("cradmin_viewhelpers_testapp.FilterTestModel", mynulldatetimefield=None)
        self.assertEqual({isnull}, set(testfilter.filter(queryobject=FilterTestModel.objects.all())))

    def test_is_not_null(self):
        testfilter = listfilter.django.single.select.NullDateTime(slug="mynulldatetimefield")
        testfilter.set_values(values=["is_not_null"])
        isnotnull = baker.make(
            "cradmin_viewhelpers_testapp.FilterTestModel",
            mynulldatetimefield=datetimeutils.default_timezone_datetime(2014, 12, 31),
        )
        baker.make("cradmin_viewhelpers_testapp.FilterTestModel", mynulldatetimefield=None)
        self.assertEqual({isnotnull}, set(testfilter.filter(queryobject=FilterTestModel.objects.all())))


class TestOrderBy(TestCase):
    def test_invalid_value(self):
        class OrderByFilter(listfilter.django.single.select.AbstractOrderBy):
            def get_ordering_options(self):
                return []

        testfilter = OrderByFilter(slug="orderby")
        testfilter.set_values(values=["invalidstuff"])
        testitem = baker.make("cradmin_viewhelpers_testapp.FilterTestModel")
        self.assertEqual({testitem}, set(testfilter.filter(queryobject=FilterTestModel.objects.all())))

    def test_no_value_defaults_to_the_empty_value(self):
        class OrderByFilter(listfilter.django.single.select.AbstractOrderBy):
            def get_ordering_options(self):
                return [
                    (
                        "",
                        {
                            "label": "mylabel",
                            "order_by": ["mydatetimefield"],
                        },
                    ),
                ]

        testfilter = OrderByFilter(slug="orderby")
        testfilter.set_values(values=[])
        testitem2 = baker.make(
            "cradmin_viewhelpers_testapp.FilterTestModel",
            mydatetimefield=datetimeutils.default_timezone_datetime(2016, 1, 2),
        )
        testitem1 = baker.make(
            "cradmin_viewhelpers_testapp.FilterTestModel",
            mydatetimefield=datetimeutils.default_timezone_datetime(2015, 1, 2),
        )
        self.assertEqual([testitem1, testitem2], list(testfilter.filter(queryobject=FilterTestModel.objects.all())))

    def test_nondefault_value(self):
        class OrderByFilter(listfilter.django.single.select.AbstractOrderBy):
            def get_ordering_options(self):
                return [
                    (
                        "",
                        {
                            "label": "asc",
                            "order_by": ["mydatetimefield"],
                        },
                    ),
                    (
                        "desc",
                        {
                            "label": "desc",
                            "order_by": ["-mydatetimefield"],
                        },
                    ),
                ]

        testfilter = OrderByFilter(slug="orderby")
        testfilter.set_values(values=["desc"])
        testitem2 = baker.make(
            "cradmin_viewhelpers_testapp.FilterTestModel",
            mydatetimefield=datetimeutils.default_timezone_datetime(2016, 1, 2),
        )
        testitem1 = baker.make(
            "cradmin_viewhelpers_testapp.FilterTestModel",
            mydatetimefield=datetimeutils.default_timezone_datetime(2015, 1, 2),
        )
        self.assertEqual([testitem2, testitem1], list(testfilter.filter(queryobject=FilterTestModel.objects.all())))



from django.core import mail
from django.test import TestCase
from cradmin_legacy.apps.cradmin_email import emailutils


class TestConvertHtmlToPlaintext(TestCase):
    def test_single_paragraph(self):
        self.assertEqual(emailutils.convert_html_to_plaintext("<p>Hello</p>").strip(), "Hello")

    def test_multiple_paragraphs(self):
        self.assertEqual(emailutils.convert_html_to_plaintext("<p>Hello</p><p>World</p>").strip(), "Hello\n\nWorld")

    def test_bold(self):
        self.assertEqual(emailutils.convert_html_to_plaintext("<strong>Hello</strong>").strip(), "**Hello**")

    def test_italic(self):
        self.assertEqual(emailutils.convert_html_to_plaintext("<em>Hello</em>").strip(), "_Hello_")

    def test_link(self):
        self.assertEqual(
            emailutils.convert_html_to_plaintext('<a href="http://example.com">Example</a>').strip(),
            "[Example](http://example.com)",
        )

    def test_link_with_classes(self):
        self.assertEqual(
            emailutils.convert_html_to_plaintext('<a href="http://example.com" class="btn">Example</a>').strip(),
            "[Example](http://example.com)",
        )


class TestAbstractEmail(TestCase):
    def test_subject(self):
        class MyEmail(emailutils.AbstractEmail):
            subject_template = "cradmin_email_testapp/abstractemail/subject.django.txt"
            html_message_template = "cradmin_email_testapp/abstractemail/html_message.django.html"

        MyEmail(recipient="test@example.com", extra_context_data={"name": "Test"}).send()
        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].subject, "Hello, Test")

    def test_subject_prefix(self):
        class MyEmail(emailutils.AbstractEmail):
            subject_template = "cradmin_email_testapp/abstractemail/subject.django.txt"
            html_message_template = "cradmin_email_testapp/abstractemail/html_message.django.html"

        with self.settings(CRADMIN_LEGACY_EMAIL_SUBJECT_PREFIX="[Testbrand] "):
            MyEmail(recipient="test@example.com", extra_context_data={"name": "Test"}).send()
        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].subject, "[Testbrand] Hello, Test")

    def test_html_message(self):
        class MyEmail(emailutils.AbstractEmail):
            subject_template = "cradmin_email_testapp/abstractemail/subject.django.txt"
            html_message_template = "cradmin_email_testapp/abstractemail/html_message.django.html"

        MyEmail(recipient="test@example.com", extra_context_data={"name": "Test"}).send()
        html_altenative = mail.outbox[0].alternatives[0]
        self.assertEqual(html_altenative[1], "text/html")
        self.assertEqual(html_altenative[0].strip(), "<p>Hello</p><p>World</p><p>Test</p>")

    def test_plaintext_message_from_html_message(self):
        class MyEmail(emailutils.AbstractEmail):
            subject_template = "cradmin_email_testapp/abstractemail/subject.django.txt"
            html_message_template = "cradmin_email_testapp/abstractemail/html_message.django.html"

        MyEmail(recipient="test@example.com", extra_context_data={"name": "Test"}).send()
        self.assertEqual(mail.outbox[0].body.strip(), "Hello\n\nWorld\n\nTest")

    def test_plaintext_message_from_template(self):
        class MyEmail(emailutils.AbstractEmail):
            subject_template = "cradmin_email_testapp/abstractemail/subject.django.txt"
            html_message_template = "cradmin_email_testapp/abstractemail/html_message.django.html"
            plaintext_message_template = "cradmin_email_testapp/abstractemail/plaintext_message.django.txt"

        MyEmail(recipient="test@example.com", extra_context_data={"name": "Test"}).send()
        self.assertEqual(mail.outbox[0].body.strip(), "Hello PlainText World Test")



from django.test import TestCase
import htmls

from cradmin_legacy.renderable import AbstractRenderableWithCss
from cradmin_legacy.viewhelpers import listbuilder
from cradmin_legacy.python2_compatibility import mock


class MinimalRenderable(AbstractRenderableWithCss):
    template_name = "cradmin_viewhelpers_testapp/listbuilder/minimal-renderable.django.html"


class TestAbstractItemRenderer(TestCase):
    def test_without_valuealias(self):
        itemrenderer = listbuilder.base.AbstractItemRenderer(value="testvalue")
        self.assertEqual("testvalue", itemrenderer.value)

    def test_with_valuealias(self):
        class MyItemRenderer(listbuilder.base.AbstractItemRenderer):
            valuealias = "myvalue"

        itemrenderer = MyItemRenderer(value="testvalue")
        self.assertEqual("testvalue", itemrenderer.value)
        self.assertEqual("testvalue", itemrenderer.myvalue)


class TestItemValueRenderer(TestCase):
    def test_render(self):
        rendered = listbuilder.base.ItemValueRenderer(value="testvalue").render()
        selector = htmls.S(rendered)
        self.assertEqual("testvalue", selector.one(".cradmin-legacy-listbuilder-itemvalue").alltext_normalized)


class TestItemFrameRenderer(TestCase):
    def test_render(self):
        rendered = listbuilder.base.ItemFrameRenderer(
            inneritem=listbuilder.base.ItemValueRenderer(value="testvalue")
        ).render(request=mock.MagicMock())
        selector = htmls.S(rendered)
        self.assertTrue(selector.exists(".cradmin-legacy-listbuilder-itemframe"))
        self.assertEqual("testvalue", selector.one(".cradmin-legacy-listbuilder-itemvalue").alltext_normalized)


class TestList(TestCase):
    def test_append(self):
        testlist = listbuilder.base.List()
        renderable = MinimalRenderable()
        testlist.append(renderable)
        self.assertEqual([renderable], testlist.renderable_list)

    def test_extend(self):
        testlist = listbuilder.base.List()
        renderable1 = MinimalRenderable()
        renderable2 = MinimalRenderable()
        testlist.extend([renderable1, renderable2])
        self.assertEqual([renderable1, renderable2], testlist.renderable_list)

    def test_extend_with_values_without_frame_renderer_class(self):
        testlist = listbuilder.base.List()
        testlist.extend_with_values(
            ["testvalue1", "testvalue2"], value_renderer_class=listbuilder.base.ItemValueRenderer
        )
        self.assertEqual("testvalue1", testlist.renderable_list[0].value)
        self.assertEqual("testvalue2", testlist.renderable_list[1].value)
        self.assertTrue(isinstance(testlist.renderable_list[0], listbuilder.base.ItemValueRenderer))
        self.assertTrue(isinstance(testlist.renderable_list[1], listbuilder.base.ItemValueRenderer))

    def test_extend_with_values_with_frame_renderer_class(self):
        testlist = listbuilder.base.List()
        testlist.extend_with_values(
            ["testvalue1", "testvalue2"],
            value_renderer_class=listbuilder.base.ItemValueRenderer,
            frame_renderer_class=listbuilder.base.ItemFrameRenderer,
        )
        self.assertEqual("testvalue1", testlist.renderable_list[0].value)
        self.assertEqual("testvalue2", testlist.renderable_list[1].value)
        self.assertTrue(isinstance(testlist.renderable_list[0], listbuilder.base.ItemFrameRenderer))
        self.assertTrue(isinstance(testlist.renderable_list[1], listbuilder.base.ItemFrameRenderer))

    def test_extend_with_values_value_and_frame_renderer_kwargs_dict(self):
        testlist = listbuilder.base.List()
        testlist.extend_with_values(
            ["testvalue1", "testvalue2"],
            value_renderer_class=listbuilder.base.ItemValueRenderer,
            value_and_frame_renderer_kwargs={"extra_kwarg": 10},
        )
        self.assertEqual({"extra_kwarg": 10}, testlist.renderable_list[0].kwargs)
        self.assertEqual({"extra_kwarg": 10}, testlist.renderable_list[1].kwargs)

    def test_extend_with_values_value_and_frame_renderer_kwargs_function(self):
        def kwargs_function(value):
            if value == "testvalue1":
                return {"is_first": True}
            else:
                return {}

        testlist = listbuilder.base.List()
        testlist.extend_with_values(
            ["testvalue1", "testvalue2"],
            value_renderer_class=listbuilder.base.ItemValueRenderer,
            value_and_frame_renderer_kwargs=kwargs_function,
        )
        self.assertEqual({"is_first": True}, testlist.renderable_list[0].kwargs)
        self.assertEqual({}, testlist.renderable_list[1].kwargs)

    def test_from_value_iterable_without_frame_renderer_class(self):
        testlist = listbuilder.base.List.from_value_iterable(
            ["testvalue1", "testvalue2"], value_renderer_class=listbuilder.base.ItemValueRenderer
        )
        self.assertEqual("testvalue1", testlist.renderable_list[0].value)
        self.assertEqual("testvalue2", testlist.renderable_list[1].value)
        self.assertTrue(isinstance(testlist.renderable_list[0], listbuilder.base.ItemValueRenderer))
        self.assertTrue(isinstance(testlist.renderable_list[1], listbuilder.base.ItemValueRenderer))

    def test_from_value_iterable_with_frame_renderer_class(self):
        testlist = listbuilder.base.List.from_value_iterable(
            ["testvalue1", "testvalue2"],
            value_renderer_class=listbuilder.base.ItemValueRenderer,
            frame_renderer_class=listbuilder.base.ItemFrameRenderer,
        )
        self.assertEqual("testvalue1", testlist.renderable_list[0].value)
        self.assertEqual("testvalue2", testlist.renderable_list[1].value)
        self.assertTrue(isinstance(testlist.renderable_list[0], listbuilder.base.ItemFrameRenderer))
        self.assertTrue(isinstance(testlist.renderable_list[1], listbuilder.base.ItemFrameRenderer))

    def test_render(self):
        testlist = listbuilder.base.List()
        testlist.append(listbuilder.base.ItemValueRenderer(value="testvalue1"))
        testlist.append(listbuilder.base.ItemValueRenderer(value="testvalue2"))
        selector = htmls.S(testlist.render(request=mock.MagicMock()))
        self.assertEqual(2, selector.count("li"))
        self.assertEqual("testvalue1", selector.list("li")[0].alltext_normalized)
        self.assertEqual("testvalue2", selector.list("li")[1].alltext_normalized)



from cradmin_legacy.crispylayouts import CradminSubmitButton


class BulkFileUploadSubmit(CradminSubmitButton):
    template = "cradmin_legacy/apps/cradmin_temporaryfileuploadstore/bulkfileupload-submit.django.html"
    extra_button_attributes = {"cradmin-legacy-bulkfileupload-submit": ""}

    def __init__(self, name, value, uploading_text=None, uploading_icon_cssclass=None, **kwargs):
        self.uploading_text = uploading_text or value
        self.uploading_icon_cssclass = uploading_icon_cssclass
        super().__init__(name, value, **kwargs)



from cradmin_legacy import crapp
from cradmin_legacy.superuserui.views import createview
from cradmin_legacy.superuserui.views import deleteview
from cradmin_legacy.superuserui.views import editview
from cradmin_legacy.superuserui.views import listview


class DjangoModelCrApp(crapp.App):
    """
    Cradmin App that provides list, create, update and delete views
    for a Django model.
    """

    #: The :class:`cradmin_legacy.superuserui.superuserui_registry.ModelConfig` this
    #: app belongs to.
    #: This is set automatically in
    #: :meth:`cradmin_legacy.superuserui.superuserui_registry.ModelConfig.make_crapp_class`
    modelconfig = None

    @classmethod
    def get_listview_class(cls):
        return listview.View

    @classmethod
    def get_foreignkeyselectview_class(cls):
        return listview.ForeignKeySelectView

    @classmethod
    def get_manytomanyselectview_class(cls):
        return listview.ManyToManySelectView

    @classmethod
    def get_editview_class(cls):
        return editview.View

    @classmethod
    def get_createview_class(cls):
        return createview.View

    @classmethod
    def get_deleteview_class(cls):
        return deleteview.View

    @classmethod
    def add_listview(cls, appurls):
        listview_class = cls.get_listview_class()
        if listview_class:
            appurls.append(crapp.Url(r"^$", listview_class.as_view(), name=crapp.INDEXVIEW_NAME))
            appurls.append(crapp.Url(r"^filter/(?P<filters_string>.+)?$", listview_class.as_view(), name="filter"))
        else:
            raise NotImplementedError(
                "You must return a view class from get_listview_class(). "
                "Note that it can be something other than a list view, "
                "such as an overview of with information/stats for the model."
            )

    @classmethod
    def add_foreignkeyselectview(cls, appurls):
        foreignkeyselectview_class = cls.get_foreignkeyselectview_class()
        if foreignkeyselectview_class:
            appurls.append(
                crapp.Url(r"^foreignkeyselect$", foreignkeyselectview_class.as_view(), name="foreignkeyselect")
            )
            appurls.append(
                crapp.Url(
                    r"^foreignkeyselect/filter/(?P<filters_string>.+)?$",
                    foreignkeyselectview_class.as_view(),
                    name="foreignkeyselect-filter",
                )
            )

    @classmethod
    def add_manytomanyselectview(cls, appurls):
        manytomanyselectview_class = cls.get_manytomanyselectview_class()
        if manytomanyselectview_class:
            appurls.append(
                crapp.Url(r"^manytomanyselect$", manytomanyselectview_class.as_view(), name="manytomanyselect")
            )
            appurls.append(
                crapp.Url(
                    r"^manytomanyselect/filter/(?P<filters_string>.+)?$",
                    manytomanyselectview_class.as_view(),
                    name="manytomanyselect-filter",
                )
            )

    @classmethod
    def add_createview(cls, appurls):
        createview_class = cls.get_createview_class()
        if createview_class:
            appurls.append(crapp.Url(r"^create/$", createview_class.as_view(), name="create"))

    @classmethod
    def add_editview(cls, appurls):
        editview_class = cls.get_editview_class()
        if editview_class:
            appurls.append(crapp.Url(r"^edit/(?P<pk>\d+)$", editview_class.as_view(), name="edit"))

    @classmethod
    def add_deleteview(cls, appurls):
        deleteview_class = cls.get_deleteview_class()
        if deleteview_class:
            appurls.append(crapp.Url(r"^delete/(?P<pk>\d+)$", deleteview_class.as_view(), name="delete"))

    @classmethod
    def get_appurls(cls):
        appurls = []
        cls.add_listview(appurls=appurls)
        cls.add_foreignkeyselectview(appurls=appurls)
        cls.add_manytomanyselectview(appurls=appurls)
        cls.add_createview(appurls=appurls)
        cls.add_editview(appurls=appurls)
        cls.add_deleteview(appurls=appurls)

        return appurls



from django.utils.translation import gettext_lazy
from django.views.generic import ListView
from django.http import HttpResponseRedirect

from cradmin_legacy.registry import cradmin_instance_registry


class RoleSelectView(ListView):
    paginate_by = 30
    template_name = "cradmin_legacy/roleselect.django.html"
    context_object_name = "roles"
    pagetitle = gettext_lazy("What would you like to edit?")
    autoredirect_if_single_role = True
    list_cssclass = "cradmin-legacy-roleselect-list cradmin-legacy-roleselect-list-flat"

    def get_queryset(self):
        cradmin_instance = cradmin_instance_registry.get_current_instance(self.request)
        return cradmin_instance.get_rolequeryset()

    def get(self, *args, **kwargs):
        cradmin_instance = cradmin_instance_registry.get_current_instance(self.request)
        if self.get_autoredirect_if_single_role() and self.get_queryset().count() == 1:
            only_role = self.get_queryset().first()
            return HttpResponseRedirect(str(cradmin_instance.rolefrontpage_url(cradmin_instance.get_roleid(only_role))))
        else:
            # Add cradmin_instance to request just like cradmin_legacy.decorators.cradminview.
            # Convenient when overriding standalone-base.django.html and using the current
            # CrInstance to distinguish multiple crinstances.
            self.request.cradmin_instance = cradmin_instance
            return super().get(*args, **kwargs)

    def get_autoredirect_if_single_role(self):
        return self.autoredirect_if_single_role

    def get_pagetitle(self):
        return self.pagetitle

    def get_list_cssclass(self):
        return self.list_cssclass

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["pagetitle"] = self.get_pagetitle()
        context["list_cssclass"] = self.get_list_cssclass()
        return context



from django.conf import settings
from django.contrib.auth import logout


def cradmin_logoutview(request, template_name="cradmin_authenticate/logout.django.html"):
    next_page = None
    if "next" in request.GET:
        next_page = request.GET["next"]
    return logout(
        request, template_name=template_name, next_page=next_page, extra_context={"LOGIN_URL": settings.LOGIN_URL}
    )



from django.conf import settings


def get_setting(settingvariable, fallback=None):
    return getattr(settings, settingvariable, fallback)


def get_cradmin_legacy_menu_scroll_top_fixed_setting():
    scroll_top_fixed_settings = get_setting("CRADMIN_LEGACY_MENU_SCROLL_TOP_FIXED", False)
    if scroll_top_fixed_settings is True:
        return {
            "cssClasses": {
                "scrollingClass": "cradmin-legacy-menu-scrolling",
            }
        }
    else:
        return False


def cradmin(request):
    return {
        "CRADMIN_LEGACY_THEME_PATH": get_setting(
            "CRADMIN_LEGACY_THEME_PATH", "cradmin_legacy/dist/css/cradmin_theme_default/theme.css"
        ),
        "CRADMIN_LEGACY_MENU_SCROLL_TOP_FIXED": get_cradmin_legacy_menu_scroll_top_fixed_setting(),
        "CRADMIN_LEGACY_MOMENTJS_LOCALE": get_setting("CRADMIN_LEGACY_MOMENTJS_LOCALE", None),
        "CRADMIN_LEGACY_CSS_ICON_LIBRARY_PATH": get_setting(
            "CRADMIN_LEGACY_CSS_ICON_LIBRARY_PATH",
            "cradmin_legacy/dist/vendor/fonts/fontawesome/css/font-awesome.min.css",
        ),
        "CRADMIN_LEGACY_HIDE_PAGE_HEADER": get_setting("CRADMIN_LEGACY_HIDE_PAGE_HEADER", False),
    }



from django.contrib.auth import get_user_model
from django.urls import reverse
from django.test import TestCase
import htmls

from django_cradmin.apps.cradmin_generic_token_with_metadata.models import GenericTokenWithMetadata
from cradmin_legacy.tests.helpers import create_user


class TestBeginRegisterAccountView(TestCase):
    def setUp(self):
        self.url = reverse("cradmin-register-account-begin")

    def test_get(self):
        response = self.client.get(self.url)
        selector = htmls.S(response.content)
        self.assertTrue(selector.exists("form#cradmin_legacy_register_account_form"))
        self.assertEqual(selector.one("h1").alltext_normalized, "Create your Testsite account")
        self.assertEqual(selector.one("title").alltext_normalized, "Create your Testsite account")
        self.assertTrue(selector.exists('input[type="email"][name="email"]'))
        self.assertTrue(selector.exists('input[type="text"][name="username"]'))
        self.assertTrue(selector.exists('input[type="password"][name="password1"]'))
        self.assertTrue(selector.exists('input[type="password"][name="password2"]'))
        self.assertEqual(selector.one('button[type="submit"]').alltext_normalized, "Sign up for Testsite")

    def test_post_email_not_unique(self):
        create_user("testuser", email="testuser@example.com")
        response = self.client.post(
            self.url, {"username": "test", "password1": "test", "password2": "test", "email": "testuser@example.com"}
        )
        self.assertEqual(response.status_code, 200)
        selector = htmls.S(response.content)
        self.assertIn(
            "Account with this email address already exists",
            selector.one("form#cradmin_legacy_register_account_form").alltext_normalized,
        )

    def test_post_passwords_not_matching(self):
        response = self.client.post(
            self.url, {"username": "test", "password1": "test1", "password2": "test2", "email": "test@example.com"}
        )
        self.assertEqual(response.status_code, 200)
        selector = htmls.S(response.content)
        self.assertIn(
            "The passwords do not match", selector.one("form#cradmin_legacy_register_account_form").alltext_normalized
        )

    def test_post_creates_token(self):
        self.assertEqual(GenericTokenWithMetadata.objects.count(), 0)
        self.client.post(
            self.url, {"username": "test", "password1": "test", "password2": "test", "email": "test@example.com"}
        )
        self.assertEqual(GenericTokenWithMetadata.objects.count(), 1)
        token = GenericTokenWithMetadata.objects.first()
        self.assertIsNotNone(token.expiration_datetime)

    def test_post_creates_user(self):
        self.assertEqual(get_user_model().objects.count(), 0)
        self.client.post(
            self.url, {"username": "test", "password1": "test", "password2": "test", "email": "test@example.com"}
        )
        self.assertEqual(get_user_model().objects.count(), 1)
        created_user = get_user_model().objects.first()
        self.assertEqual(created_user.username, "test")
        self.assertEqual(created_user.email, "test@example.com")
        self.assertTrue(created_user.has_usable_password())
        self.assertTrue(created_user.check_password("test"))

    def test_post_next_url_as_querystring_argument(self):
        self.assertEqual(get_user_model().objects.count(), 0)
        self.client.post(
            f"{self.url}?next=/next",
            {"username": "test", "password1": "test", "password2": "test", "email": "test@example.com"},
        )
        token = GenericTokenWithMetadata.objects.first()
        self.assertEqual(token.metadata["next_url"], "/next")

    def test_post_next_url_as_cradmin_legacy_register_account_redirect_url_setting(self):
        self.assertEqual(get_user_model().objects.count(), 0)
        with self.settings(CRADMIN_LEGACY_REGISTER_ACCOUNT_REDIRECT_URL="/redirect"):
            self.client.post(
                self.url, {"username": "test", "password1": "test", "password2": "test", "email": "test@example.com"}
            )
        token = GenericTokenWithMetadata.objects.first()
        self.assertEqual(token.metadata["next_url"], "/redirect")

    def test_post_next_url_as_login_url_setting(self):
        self.assertEqual(get_user_model().objects.count(), 0)
        with self.settings(LOGIN_URL="/login"):
            self.client.post(
                self.url, {"username": "test", "password1": "test", "password2": "test", "email": "test@example.com"}
            )
        token = GenericTokenWithMetadata.objects.first()
        self.assertEqual(token.metadata["next_url"], "/login")



from django.test import TestCase
from cradmin_legacy.apps.cradmin_register_account.forms.email import EmailUserCreateAccountForm
from cradmin_legacy.apps.cradmin_register_account.tests.cradmin_register_account_testapp.models import EmailUser


class EmailUserCreateAccountFormStaticUserModel(EmailUserCreateAccountForm):
    class Meta(EmailUserCreateAccountForm.Meta):
        model = EmailUser


class TestEmailUserCreateAccountForm(TestCase):
    def test_is_valid(self):
        form = EmailUserCreateAccountFormStaticUserModel(
            {"password1": "test", "password2": "test", "email": "test@example.com"}
        )
        self.assertTrue(form.is_valid())

    def test_save(self):
        form = EmailUserCreateAccountFormStaticUserModel(
            {"password1": "test", "password2": "test", "email": "test@example.com"}
        )
        form.is_valid()
        self.assertEqual(EmailUser.objects.count(), 0)
        form.save()
        self.assertEqual(EmailUser.objects.count(), 1)
        created_user = EmailUser.objects.first()
        self.assertEqual(created_user.email, "test@example.com")
        self.assertTrue(created_user.has_usable_password())
        self.assertTrue(created_user.check_password("test"))

    def test_passwords_do_not_match(self):
        form = EmailUserCreateAccountFormStaticUserModel(
            {"password1": "test1", "password2": "test2", "email": "unused@example.com"}
        )
        self.assertFalse(form.is_valid())
        self.assertEqual(form.non_field_errors(), ["The passwords do not match."])

    def test_email_is_not_unique(self):
        EmailUser.objects.create(email="test@example.com")
        form = EmailUserCreateAccountFormStaticUserModel(
            {"password1": "unused", "password2": "unused", "email": "test@example.com"}
        )
        self.assertFalse(form.is_valid())
        self.assertEqual(form.errors, {"email": ["Email user with this Email already exists."]})



from functools import wraps

from django.conf import settings
from django.contrib.auth import REDIRECT_FIELD_NAME
from django.core.exceptions import ObjectDoesNotExist
from django.http import HttpResponse
from django.utils.encoding import force_str
from django.shortcuts import resolve_url

from cradmin_legacy.registry import cradmin_instance_registry


def has_access_to_cradmin_instance(view_function, redirect_field_name=REDIRECT_FIELD_NAME, login_url=None):
    """
    Decorator for cradmin_legacy views.

    Makes it impossible to access the view unless the
    :meth:`cradmin_legacy.crinstance.BaseCrAdminInstance.has_access` method
    returns ``True``.

    Adds the following variables to the request:

        - ``cradmin_instance``: The detected cradmin instance.
    """

    @wraps(view_function)
    def wrapper(request, *args, **kwargs):
        cradmin_instance = cradmin_instance_registry.get_current_instance(request)
        if cradmin_instance.has_access():
            request.cradmin_instance = cradmin_instance
            return view_function(request, *args, **kwargs)
        else:
            # Redirect to login just like login_required()
            from django.contrib.auth.views import redirect_to_login

            path = request.build_absolute_uri()
            resolved_login_url = force_str(resolve_url(login_url or settings.LOGIN_URL))
            return redirect_to_login(path, resolved_login_url, redirect_field_name)

    return wrapper


def cradminview(view_function):
    """
    Decorator for cradmin_legacy views.

    Protects the view, making it impossible to access unless the requesting user
    has the role defined by the named url variable ``roleid``.

    Adds the following variables to the request:

        - ``cradmin_role``: The detected cradmin role. This is ``None``
            if the ``roleclass`` attribute of the cradmin instance is ``None``.
    """

    @wraps(view_function)
    def wrapper(request, *args, **kwargs):
        if request.cradmin_instance.roleclass:
            roleid = kwargs.pop("roleid")
            role = request.cradmin_instance.get_role_from_roleid(roleid)
            if not role:
                return request.cradmin_instance.invalid_roleid_response(roleid)
            try:
                role_from_rolequeryset = request.cradmin_instance.get_role_from_rolequeryset(role)
            except ObjectDoesNotExist:
                response = request.cradmin_instance.missing_role_response(role)
            else:
                request.cradmin_role = role_from_rolequeryset
                response = view_function(request, *args, **kwargs)
        else:
            request.cradmin_role = None
            response = view_function(request, *args, **kwargs)

        if isinstance(response, HttpResponse):
            http_headers = request.cradmin_instance.get_common_http_headers()
            if http_headers:
                for headerattribute, headervalue in http_headers.items():
                    response[headerattribute] = headervalue

        request.cradmin_instance.add_extra_instance_variables_to_request(request)
        return response

    return wrapper



from django.core.exceptions import ValidationError
from django.test import TestCase
from cradmin_legacy.formfields.email_list import EmailListField


class TestEmailListField(TestCase):
    def test_to_python_none(self):
        self.assertEqual(EmailListField().to_python(None), [])

    def test_to_python_empty_list(self):
        self.assertEqual(EmailListField().to_python([]), [])

    def test_to_python_empty_tuple(self):
        self.assertEqual(EmailListField().to_python(()), [])

    def test_to_python_list(self):
        self.assertEqual(
            EmailListField().to_python(["a@example.com", "b@example.com"]), ["a@example.com", "b@example.com"]
        )

    def test_to_python_tuple(self):
        self.assertEqual(
            EmailListField().to_python(("a@example.com", "b@example.com")), ["a@example.com", "b@example.com"]
        )

    def test_to_python_string(self):
        self.assertEqual(EmailListField().to_python("a@example.com b@example.com"), ["a@example.com", "b@example.com"])

    def test_to_python_empty_string(self):
        self.assertEqual(EmailListField().to_python(""), [])

    def test_to_python_string_with_spaces_only(self):
        self.assertEqual(EmailListField().to_python("     "), [])

    def test_to_python_string_with_spaces_at_beginning(self):
        self.assertEqual(EmailListField().to_python("     x@example.com"), ["x@example.com"])

    def test_to_python_string_with_spaces_at_end(self):
        self.assertEqual(EmailListField().to_python("x@example.com    "), ["x@example.com"])

    def test_to_python_invalid_email(self):
        with self.assertRaisesMessage(ValidationError, "Invalid email address: b"):
            EmailListField().to_python("a@example.com b c")



#!/usr/bin/env python

from vitalstyles.settings import Settings
from vitalstyles.cli import cli


class CustomSettings(Settings):
    def setup_without_settingsfile(self):
        self.settings.update({
            "preview_cssfile": "styles.css",
            "title": "cradmin_legacy styles",
            "inpaths": [
                "src/less"
            ],
            # "asset_directories": [
            #     "styleguide_assets"
            # ],
            'include_stock_assets': True
        })

if __name__ == '__main__':
    cli(settingsobject=CustomSettings())



from django.contrib import admin
from django.template import defaultfilters

from cradmin_legacy.apps.cradmin_imagearchive.models import ArchiveImage


class ArchiveImageAdmin(admin.ModelAdmin):
    list_display = [
        "id",
        "name",
        "description",
        "role",
        "image_width",
        "image_height",
        "get_size",
        "get_url",
    ]
    list_display_links = [
        "id",
        "name",
    ]

    search_fields = [
        "id",
        "name",
        "description",
    ]

    readonly_fields = [
        "image_width",
        "image_height",
        "file_extension",
    ]

    def get_queryset(self, request):
        if request.user.is_superuser:
            return super().get_queryset(request)
        else:
            return ArchiveImage.objects.none()

    def get_size(self, obj):
        if obj.file_size:
            return defaultfilters.filesizeformat(obj.file_size)
        else:
            return None

    get_size.short_description = "Size"
    get_size.admin_order_field = "file_size"

    def get_url(self, obj):
        return '<a href="{url}" target="_blank">{label}</a>'.format(label="Show image", url=obj.image.url)

    get_url.short_description = "Open"
    get_url.allow_tags = True


admin.site.register(ArchiveImage, ArchiveImageAdmin)



from io import BytesIO

from PIL import Image, ImageDraw


def create_image(width, height, filetype="PNG"):
    """
    Create an image of the given ``width``, ``height`` and optional ``filetype``.

    Returns:
        A bytestring with the generated image.
    """
    im = Image.new("RGB", (width, height), color="#191919")
    ImageDraw.Draw(im)
    out = BytesIO()
    im.save(out, filetype)
    return out.getvalue()



from django.contrib.auth.models import AbstractBaseUser, BaseUserManager
from django.db import models


class EmailUser(AbstractBaseUser):
    USERNAME_FIELD = "email"
    REQUIRED_FIELDS = []
    objects = BaseUserManager()

    email = models.EmailField(max_length=250, blank=False, unique=True)

    def get_short_name(self):
        return self.email

    def get_full_name(self):
        return self.email



import htmls
from unittest import mock
from django import test
from django.http import QueryDict

from cradmin_legacy.viewhelpers import listbuilder
from cradmin_legacy.viewhelpers.multiselect2 import manytomanyview


class TestViewMixin(test.TestCase):
    def test_get_selected_values_list(self):
        view = manytomanyview.ViewMixin()
        view.request = mock.MagicMock()
        view.request.GET = QueryDict("", mutable=True)
        view.request.GET.update({"manytomany_select_current_value": "[1,2]"})
        self.assertEqual([1, 2], view.get_selected_values_list())

    def test_get_selected_objects(self):
        view = manytomanyview.ViewMixin()
        view.request = mock.MagicMock()
        view.request.cradmin_role = "testrole"
        view.get_selected_values_list = mock.MagicMock(return_value="testvalue")
        mockqueryset = mock.MagicMock()
        view.get_queryset_for_role = mock.MagicMock(return_value=mockqueryset)

        view.get_selected_objects()
        mockqueryset.filter.assert_called_once_with(pk__in="testvalue")
        view.get_queryset_for_role.assert_called_once_with("testrole")

    def test_get_selected_objects_has_get_unfiltered_queryset_for_role_method(self):
        view = manytomanyview.ViewMixin()
        view.request = mock.MagicMock()
        view.request.cradmin_role = "testrole"
        view.get_selected_values_list = mock.MagicMock(return_value="testvalue")
        view.get_queryset_for_role = mock.MagicMock()
        mockqueryset = mock.MagicMock()
        view.get_unfiltered_queryset_for_role = mock.MagicMock(return_value=mockqueryset)

        view.get_selected_objects()
        mockqueryset.filter.assert_called_once_with(pk__in="testvalue")
        view.get_unfiltered_queryset_for_role.assert_called_once_with("testrole")
        view.get_queryset_for_role.assert_not_called()

    def test_should_include_previously_selected_no_paging_no_get_filters_string(self):
        view = manytomanyview.ViewMixin()
        self.assertTrue(view.should_include_previously_selected())

    def test_should_include_previously_selected_get_filters_string_empty_string(self):
        view = manytomanyview.ViewMixin()
        view.get_filters_string = mock.MagicMock(return_value="")
        self.assertTrue(view.should_include_previously_selected())

    def test_should_include_previously_selected_get_filters_string_nonempty_string(self):
        view = manytomanyview.ViewMixin()
        view.get_filters_string = mock.MagicMock(return_value="test")
        self.assertFalse(view.should_include_previously_selected())

    def test_should_include_previously_selected_paginate_by_no_page_specified(self):
        view = manytomanyview.ViewMixin()
        view.paginate_by = 40
        view.page_kwarg = "page"
        view.kwargs = {}
        view.request = mock.MagicMock()
        view.request.GET = {}
        self.assertTrue(view.should_include_previously_selected())

    def test_should_include_previously_selected_paginate_by_page1_kwargs(self):
        view = manytomanyview.ViewMixin()
        view.paginate_by = 40
        view.page_kwarg = "page"
        view.kwargs = {"page": 1}
        view.request = mock.MagicMock()
        view.request.GET = {}
        self.assertTrue(view.should_include_previously_selected())

    def test_should_include_previously_selected_paginate_by_page1_requestget(self):
        view = manytomanyview.ViewMixin()
        view.paginate_by = 40
        view.page_kwarg = "page"
        view.kwargs = {}
        view.request = mock.MagicMock()
        view.request.GET = {"page": 1}
        self.assertTrue(view.should_include_previously_selected())

    def test_should_include_previously_selected_paginate_by_page2_kwargs(self):
        view = manytomanyview.ViewMixin()
        view.paginate_by = 40
        view.page_kwarg = "page"
        view.kwargs = {"page": 2}
        view.request = mock.MagicMock()
        view.request.GET = {}
        self.assertFalse(view.should_include_previously_selected())

    def test_should_include_previously_selected_paginate_by_page2_requestget(self):
        view = manytomanyview.ViewMixin()
        view.paginate_by = 40
        view.page_kwarg = "page"
        view.kwargs = {}
        view.request = mock.MagicMock()
        view.request.GET = {"page": 2}
        self.assertFalse(view.should_include_previously_selected())

    def test_get_queryset_for_role_should_include_previously_selected_false(self):
        mockqueryset = mock.MagicMock()

        class SuperMyView:
            def get_queryset_for_role(self, role):
                return mockqueryset

        class MyView(manytomanyview.ViewMixin, SuperMyView):
            pass

        view = MyView()
        view.request = mock.MagicMock()
        view.should_include_previously_selected = mock.MagicMock(return_value=False)
        view.get_queryset_for_role()
        mockqueryset.exclude.assert_not_called()

    def test_get_queryset_for_role_should_include_previously_selected_true(self):
        mockqueryset = mock.MagicMock()

        class SuperMyView:
            def get_queryset_for_role(self, role):
                return mockqueryset

        class MyView(manytomanyview.ViewMixin, SuperMyView):
            pass

        view = MyView()
        view.request = mock.MagicMock()
        view.should_include_previously_selected = mock.MagicMock(return_value=True)
        view.get_selected_values_list = mock.MagicMock(return_value="testvalue")
        view.get_queryset_for_role()
        mockqueryset.exclude.assert_called_once_with(pk__in="testvalue")

    def test_get_target_renderer_class(self):
        view = manytomanyview.ViewMixin()
        self.assertEqual(manytomanyview.ViewMixin.target_renderer_class, view.get_target_renderer_class())

    def test_get_target_renderer_kwargs(self):
        view = manytomanyview.ViewMixin()
        view.request = mock.MagicMock()
        view.request.GET = {"manytomany_select_fieldid": "testid", "manytomany_select_required": "True"}
        target_renderer_kwargs = view.get_target_renderer_kwargs()
        self.assertEqual("testid", target_renderer_kwargs["target_formfield_id"])
        self.assertFalse(target_renderer_kwargs["empty_selection_allowed"])

    def test_get_target_renderer(self):
        view = manytomanyview.ViewMixin()
        view.request = mock.MagicMock()
        view.request.GET = {"manytomany_select_fieldid": "testid", "manytomany_select_required": "True"}
        target_renderer = view.get_target_renderer()
        self.assertEqual("testid", target_renderer.target_formfield_id)


class TestListBuilderViewMixin(test.TestCase):
    def test_get_listbuilder_list_should_include_previously_selected_false(self):
        class SuperMyView:
            def get_listbuilder_list(self, context):
                return listbuilder.base.List()

        class MyView(manytomanyview.ListBuilderViewMixin, SuperMyView):
            pass

        view = MyView()
        view.should_include_previously_selected = mock.MagicMock(return_value=False)
        self.assertFalse(view.get_listbuilder_list(context={}).has_items())

    def test_get_listbuilder_list_should_include_previously_selected_true(self):
        class SuperMyView:
            def get_listbuilder_list(self, context):
                return listbuilder.base.List()

        class MyView(manytomanyview.ListBuilderViewMixin, SuperMyView):
            pass

        view = MyView()
        view.should_include_previously_selected = mock.MagicMock(return_value=True)
        view.get_value_renderer_class = mock.MagicMock(return_value=listbuilder.base.ItemValueRenderer)
        view.get_frame_renderer_class = mock.MagicMock(return_value=listbuilder.base.ItemFrameRenderer)
        view.get_selected_objects = mock.MagicMock(return_value=["testvalue"])
        selector = htmls.S(view.get_listbuilder_list(context={}).render())
        self.assertTrue("testvalue", selector.one(".cradmin-legacy-listbuilder-itemvalue").alltext_normalized)


class ListBuilderFilterListViewMixin(test.TestCase):
    def test_get_filterlist_url(self):
        view = manytomanyview.ListBuilderFilterListViewMixin()
        view.request = mock.MagicMock()
        view.get_filterlist_url(filters_string="test")
        view.request.cradmin_app.reverse_appurl.assert_called_once_with(
            "manytomanyselect-filter", kwargs={"filters_string": "test"}
        )

    def test_add_target_renderer_to_filterlist(self):
        view = manytomanyview.ListBuilderFilterListViewMixin()
        view.get_target_renderer = mock.MagicMock(return_value="testrenderer")
        mockfilterlist = mock.MagicMock()
        view.add_target_renderer_to_filterlist(filterlist=mockfilterlist)
        mockfilterlist.append.assert_called_once_with("testrenderer")

    def test_add_filterlist_items(self):
        class SuperMyView:
            def add_filterlist_items(self, filterlist):
                pass

        class MyView(manytomanyview.ListBuilderFilterListViewMixin, SuperMyView):
            pass

        view = MyView()
        view.add_target_renderer_to_filterlist = mock.MagicMock()
        view.add_filterlist_items("testfilterlist")
        view.add_target_renderer_to_filterlist.assert_called_once_with(filterlist="testfilterlist")



from datetime import timedelta, datetime

from django.contrib.auth.models import AnonymousUser
from django.core.exceptions import PermissionDenied
from django.http import HttpResponse

from django.test import TestCase, RequestFactory
from django.utils import timezone
import htmls
from cradmin_legacy.python2_compatibility import mock

from django_cradmin.apps.cradmin_generic_token_with_metadata.models import GenericTokenWithMetadata
from cradmin_legacy.apps.cradmin_invite.baseviews.accept import AbstractAcceptInviteView
from cradmin_legacy.tests.helpers import create_user


class AcceptInviteView(AbstractAcceptInviteView):
    def get_appname(self):
        return "testapp"

    def invite_accepted(self, token):
        return HttpResponse("OK")


class TestAbstractAcceptInviteView(TestCase):
    def setUp(self):
        self.factory = RequestFactory()
        # request._messages = mock.MagicMock()

    def __create_token(self, metadata=None, expiration_datetime=None, **kwargs):
        generic_token_with_metadata = GenericTokenWithMetadata(
            created_datetime=timezone.now(),
            expiration_datetime=(expiration_datetime or (timezone.now() + timedelta(days=2))),
            content_object=create_user("invitecontentobject"),
            app="testapp",
            **kwargs,
        )
        generic_token_with_metadata.metadata = metadata or {}
        generic_token_with_metadata.save()
        return generic_token_with_metadata

    def test_get_expired_token(self):
        self.__create_token(token="valid-token", expiration_datetime=datetime(2014, 1, 1))
        request = self.factory.get("/test")
        response = AcceptInviteView.as_view()(request, token="valid-token")
        selector = htmls.S(response.content)
        self.assertTrue(selector.exists("#cradmin_legacy_invite_expired_message"))
        self.assertEqual(
            selector.one("#cradmin_legacy_invite_expired_message").alltext_normalized, "This invite link has expired."
        )

    def test_get_invalid_token(self):
        self.__create_token(token="valid-token")
        request = self.factory.get("/test")
        response = AcceptInviteView.as_view()(request, token="invalid-token")
        selector = htmls.S(response.content)
        self.assertTrue(selector.exists("#cradmin_legacy_invite_invalid_token_message"))
        self.assertEqual(
            selector.one("#cradmin_legacy_invite_invalid_token_message").alltext_normalized,
            "Invalid invite URL. Are you sure you copied the entire URL from the email?",
        )

    def test_get_render(self):
        request = self.factory.get("/test")
        request.user = mock.MagicMock()
        token = self.__create_token()
        response = AcceptInviteView.as_view()(request, token=token.token)
        self.assertEqual(response.status_code, 200)
        response.render()
        selector = htmls.S(response.content)
        self.assertEqual(selector.one("title").alltext_normalized, "Accept invite")
        self.assertEqual(selector.one(".page-header h1").alltext_normalized, "Accept invite")
        self.assertEqual(
            selector.one("#cradmin_legacy_invite_accept_description").alltext_normalized,
            "TODO: Set a description_template for your AbstractAcceptInviteView subclass.",
        )

    def test_get_render_is_authenticated(self):
        request = self.factory.get("/test")
        request.user = create_user("testuser")
        token = self.__create_token()
        with self.settings(CRADMIN_LEGACY_SITENAME="Testsite"):
            response = AcceptInviteView.as_view()(request, token=token.token)
        self.assertEqual(response.status_code, 200)
        response.render()
        selector = htmls.S(response.content)
        self.assertTrue(selector.exists("button#cradmin_legacy_invite_accept_as_button"))
        self.assertEqual(
            selector.one("#cradmin_legacy_invite_accept_register_account_button")["href"],
            "/cradmin_register_account/begin?next=http%3A%2F%2Ftestserver%2Ftest",
        )
        self.assertEqual(
            selector.one("#cradmin_legacy_invite_accept_login_as_different_user_button")["href"],
            "/cradmin_authenticate/logout?next=%2Faccounts%2Flogin%2F%3Fnext%3Dhttp%253A%252F%252Ftestserver%252Ftest",
        )

        self.assertEqual(
            selector.one("button#cradmin_legacy_invite_accept_as_button").alltext_normalized, "Accept as testuser"
        )
        self.assertEqual(
            selector.one("#cradmin_legacy_invite_accept_register_account_button").alltext_normalized,
            "Sign up for Testsite",
        )
        self.assertEqual(
            selector.one("#cradmin_legacy_invite_accept_login_as_different_user_button").alltext_normalized,
            "Sign in as another user",
        )

    def test_get_render_not_authenticated(self):
        request = self.factory.get("/test")
        request.user = AnonymousUser()
        token = self.__create_token()
        with self.settings(CRADMIN_LEGACY_SITENAME="Testsite"):
            response = AcceptInviteView.as_view()(request, token=token.token)
        self.assertEqual(response.status_code, 200)
        response.render()
        selector = htmls.S(response.content)
        self.assertEqual(
            selector.one("#cradmin_legacy_invite_accept_login_button")["href"],
            "/accounts/login/?next=http%3A%2F%2Ftestserver%2Ftest",
        )
        self.assertEqual(
            selector.one("#cradmin_legacy_invite_accept_register_account_button")["href"],
            "/cradmin_register_account/begin?next=http%3A%2F%2Ftestserver%2Ftest",
        )

        self.assertEqual(selector.one("#cradmin_legacy_invite_accept_login_button").alltext_normalized, "Sign in")
        self.assertEqual(
            selector.one("#cradmin_legacy_invite_accept_register_account_button").alltext_normalized,
            "Sign up for Testsite",
        )

    def test_post_not_authenticated(self):
        request = self.factory.post("/test")
        request.user = AnonymousUser()
        token = self.__create_token()
        with self.assertRaises(PermissionDenied):
            AcceptInviteView.as_view()(request, token=token.token)

    def test_post_is_authenticated(self):
        request = self.factory.post("/test")
        request.user = create_user("testuser")
        token = self.__create_token()
        response = AcceptInviteView.as_view()(request, token=token.token)
        self.assertEqual(response.content, b"OK")



import os
from django.core.exceptions import ValidationError
from django.core.files.base import ContentFile
from django.test import TestCase
from model_bakery import baker
from cradmin_legacy.apps.cradmin_temporaryfileuploadstore.models import (
    TemporaryFileCollection,
    TemporaryFile,
    html_input_accept_match,
    truncate_filename,
    make_unique_filename,
)
from cradmin_legacy.tests.helpers import create_user


class TestModels(TestCase):
    def test_delete_removes_physical_file(self):
        collection = TemporaryFileCollection.objects.create(user=create_user("testuser"))
        temporaryfile = TemporaryFile(filename="test.txt", collection=collection)
        temporaryfile.file.save("test.txt", ContentFile("Testdata"))
        physical_file_path = temporaryfile.file.path
        self.assertTrue(os.path.exists(physical_file_path))
        temporaryfile.file.delete()
        temporaryfile.delete()
        self.assertFalse(os.path.exists(physical_file_path))

    def test_clear_collection(self):
        collection = TemporaryFileCollection.objects.create(user=create_user("testuser"))
        temporaryfile = TemporaryFile(filename="test.txt", collection=collection)
        temporaryfile.file.save("test.txt", ContentFile("Testdata"))
        physical_file_path = temporaryfile.file.path
        self.assertTrue(os.path.exists(physical_file_path))
        collection.clear_files()
        self.assertFalse(os.path.exists(physical_file_path))
        self.assertEqual(collection.files.count(), 0)

    def test_singlemode_keeps_only_last_file(self):
        collection = TemporaryFileCollection.objects.create(user=create_user("testuser"), singlemode=True)

        first_added_temporary_file = TemporaryFile(filename="test1.txt", collection=collection)
        first_added_temporary_file.file.save("test1.txt", ContentFile("Testdata"), save=False)
        first_added_temporary_file.clean()
        first_added_temporary_file.save()

        last_added_temporary_file = TemporaryFile(filename="test2.txt", collection=collection)
        last_added_temporary_file.file.save("test1.txt", ContentFile("Testdata"), save=False)
        last_added_temporary_file.clean()
        last_added_temporary_file.save()
        self.assertEqual(collection.files.count(), 1)
        self.assertEqual(collection.files.first(), last_added_temporary_file)

    def test_singlemode_keeps_only_last_file_delete_physical_file(self):
        collection = TemporaryFileCollection.objects.create(user=create_user("testuser"), singlemode=True)

        first_added_temporary_file = TemporaryFile(filename="test1.txt", collection=collection)
        first_added_temporary_file.file.save("test1.txt", ContentFile("Testdata"), save=False)
        first_added_temporary_file.clean()
        first_added_temporary_file.save()
        first_added_physical_file_path = first_added_temporary_file.file.path
        self.assertTrue(os.path.exists(first_added_physical_file_path))

        last_added_temporary_file = TemporaryFile(filename="test2.txt", collection=collection)
        last_added_temporary_file.file.save("test1.txt", ContentFile("Testdata"), save=False)
        last_added_temporary_file.clean()
        self.assertFalse(os.path.exists(first_added_physical_file_path))

    def test_singlemode_keeps_only_last_file_do_not_delete_last(self):
        collection = TemporaryFileCollection.objects.create(user=create_user("testuser"), singlemode=True)

        last_added_temporary_file = TemporaryFile(filename="test2.txt", collection=collection)
        last_added_temporary_file.file.save("test1.txt", ContentFile("Testdata"), save=False)
        last_added_temporary_file.clean()
        last_added_temporary_file.save()
        last_added_temporary_file.clean()
        self.assertEqual(collection.files.count(), 1)
        self.assertEqual(collection.files.first(), last_added_temporary_file)

    def test_get_filename_set(self):
        collection = TemporaryFileCollection.objects.create(user=create_user("testuser"))
        temporaryfile1 = TemporaryFile(filename="test.txt", collection=collection)
        temporaryfile1.file.save("x", ContentFile("Testdata"))
        temporaryfile2 = TemporaryFile(filename="test2.txt", collection=collection)
        temporaryfile2.file.save("y", ContentFile("Testdata"))
        temporaryfile3 = TemporaryFile(filename="test.txt", collection=collection)
        temporaryfile3.file.save("z", ContentFile("Testdata"))
        self.assertEqual(collection.get_filename_set(), {"test.txt", "test2.txt"})

    def test_set_mimetype_from_filename(self):
        temporaryfile = TemporaryFile(filename="test.png")
        temporaryfile.set_mimetype_from_filename()
        self.assertEqual(temporaryfile.mimetype, "image/png")

    def test_set_mimetype_from_filename_no_extension(self):
        temporaryfile = TemporaryFile(filename="test")
        temporaryfile.set_mimetype_from_filename()
        self.assertEqual(temporaryfile.mimetype, "")

    def test_set_mimetype_from_filename_unknown_extension(self):
        temporaryfile = TemporaryFile(filename="test.thisdoesnotexist")
        temporaryfile.set_mimetype_from_filename()
        self.assertEqual(temporaryfile.mimetype, "")

    def clean_sets_mimetype(self):
        temporaryfile = TemporaryFile(filename="test.png")
        temporaryfile.clean()
        self.assertEqual(temporaryfile.mimetype, "image/png")

    def clean_truncates_filename(self):
        collection = TemporaryFileCollection(max_filename_length=6)
        temporaryfile = TemporaryFile(filename="test.png", collection=collection)
        temporaryfile.clean()
        self.assertEqual(temporaryfile.filename, "st.png")

    def clean_does_not_truncate_filename_if_max_filename_length_is_none(self):
        collection = TemporaryFileCollection(max_filename_length=None)
        temporaryfile = TemporaryFile(filename="test.png", collection=collection)
        temporaryfile.clean()
        self.assertEqual(temporaryfile.filename, "test.png")

    def clean_validates_accept(self):
        collection = TemporaryFileCollection.objects.create(user=create_user("testuser"), accept="image/png,image/jpeg")
        TemporaryFile(filename="test.png", collection=collection).clean()
        TemporaryFile(filename="test.jpg", collection=collection).clean()
        with self.assertRaises(ValidationError):
            TemporaryFile(filename="test.txt", collection=collection).clean()

    def test_html_input_accept_match_filename_star(self):
        self.assertTrue(html_input_accept_match(accept="*.jpg", mimetype="unused", filename="test.jpg"))
        self.assertFalse(html_input_accept_match(accept="*.jpg", mimetype="unused", filename="test.jpeg"))
        self.assertTrue(html_input_accept_match(accept=".unused,*.jpg,.unused", mimetype="unused", filename="test.jpg"))

    def test_html_input_accept_match_filename_dot(self):
        self.assertTrue(html_input_accept_match(accept=".jpg", mimetype="unused", filename="test.jpg"))
        self.assertFalse(html_input_accept_match(accept=".jpg", mimetype="unused", filename="test.jpeg"))
        self.assertTrue(html_input_accept_match(accept=".unused,.jpg,.unused", mimetype="unused", filename="test.jpg"))

    def test_html_input_accept_match_mimetype_exact(self):
        self.assertTrue(html_input_accept_match(accept="image/jpeg", mimetype="image/jpeg", filename="unused"))
        self.assertFalse(html_input_accept_match(accept="image/png", mimetype="image/jpeg", filename="unused"))
        self.assertTrue(
            html_input_accept_match(accept=".unused,image/jpeg,unused2", mimetype="image/jpeg", filename="unused")
        )

    def test_html_input_accept_match_mimetype_star(self):
        self.assertTrue(html_input_accept_match(accept="image/*", mimetype="image/jpeg", filename="unused"))
        self.assertFalse(html_input_accept_match(accept="application/*", mimetype="image/jpeg", filename="unused"))
        self.assertTrue(
            html_input_accept_match(accept=".unused,image/*,.unused2", mimetype="image/jpeg", filename="unused")
        )

    def test_html_input_accept_match_mimetype_none(self):
        self.assertFalse(html_input_accept_match(accept="image/jpeg", mimetype=None, filename="test.jpeg"))

    def test_truncate_filename_not_above_max_length(self):
        self.assertEqual(truncate_filename("test.txt", maxlength=8), "test.txt")

    def test_truncate_filename_veryshort_maxlength(self):
        self.assertEqual(truncate_filename("test.txt", maxlength=5), "t.txt")

    def test_truncate_filename_ellipsis_even_filelength_more_in_tail(self):
        self.assertEqual(truncate_filename("abcdefghABCDEFGH", maxlength=14), "abcde...CDEFGH")

    def test_truncate_filename_ellipsis_odd_filelength(self):
        self.assertEqual(truncate_filename("abcdeX___Xhijkl", maxlength=13), "abcde...hijkl")

    def test_make_unique_filename_already_unique(self):
        self.assertEqual(make_unique_filename(filename_set={"test.txt"}, wanted_filename="test2.txt"), "test2.txt")

    def test_make_unique_filename(self):
        self.assertNotEqual(make_unique_filename(filename_set={"test.txt"}, wanted_filename="test.txt"), "test.txt")
        self.assertTrue(
            make_unique_filename(filename_set={"test.txt"}, wanted_filename="test.txt").endswith("-test.txt")
        )

    def test_make_unique_filename_maxlength(self):
        unique_filename = make_unique_filename(
            filename_set={"t" * 45}, wanted_filename="t" * 45, max_filename_length=45
        )
        self.assertNotEqual(unique_filename, "t" * 45)
        self.assertTrue(unique_filename.endswith("-tttttttt"))
        self.assertEqual(len(unique_filename), 45)

    def test_no_max_filesize_bytes(self):
        collection = baker.make("cradmin_temporaryfileuploadstore.TemporaryFileCollection")
        temporaryfile = TemporaryFile(filename="test.txt", collection=collection)
        temporaryfile.file.save("test.txt", ContentFile("Testdata"))
        temporaryfile.clean()  # No ValidationError

    def test_max_filesize_bytes_size_below_ok(self):
        collection = baker.make("cradmin_temporaryfileuploadstore.TemporaryFileCollection", max_filesize_bytes=100)
        temporaryfile = TemporaryFile(filename="test.txt", collection=collection)
        temporaryfile.file.save("test.txt", ContentFile("Testdata"))
        temporaryfile.clean()  # No ValidationError

    def test_max_filesize_bytes_size_above_fails(self):
        collection = baker.make("cradmin_temporaryfileuploadstore.TemporaryFileCollection", max_filesize_bytes=1)
        temporaryfile = TemporaryFile(filename="test.txt", collection=collection)
        temporaryfile.file.save("test.txt", ContentFile("Testdata"))
        with self.assertRaisesMessage(ValidationError, "Can not upload files larger than 1B."):
            temporaryfile.clean()



import posixpath

from django import forms
from django.conf import settings
from django.template.loader import render_to_string
from django.utils.safestring import mark_safe


class ImageWidget(forms.ClearableFileInput):
    """
    Clearable image widget.

    Can be made un-clearable (for required fields) using the
    ``clearable`` argument for ``__init__``.
    """

    template_name = "cradmin_legacy/filewidgets/imagewidget.django.html"

    #: Comma separated string of filetypes that we should accept.
    #: Added to the file upload field as the accept attribute.
    #: Defaults to ``"image/png,image/jpeg,image/gif"``.
    accept = "image/*"

    def __init__(self, request, attrs=None, template_name=None, clearable=True, preview_imagetype=None):
        self.request = request
        self.clearable = clearable
        if template_name:
            self.template_name = template_name
        self.preview_imagetype = preview_imagetype
        self.preview_fallback_options = {"width": 300, "height": 300}

        super().__init__(attrs)

    def get_preview_css_styles(self):
        if self.preview_imagetype:
            options = settings.CRADMIN_LEGACY_IMAGEUTILS_IMAGETYPE_MAP[self.preview_imagetype]
        else:
            options = self.preview_fallback_options
        styles = []
        if "width" in options:
            styles.append("max-width: {}px".format(options["width"]))
        else:
            styles.append("max-width: 100%".format(options["width"]))
        return styles

    def build_preview_url(self, imagepath):
        if imagepath:
            return posixpath.join(settings.MEDIA_URL, imagepath)
        else:
            return None

    def get_context_data(self, imageurl, name, value, attrs):
        return {
            "imageurl": imageurl,
            "clear_checkbox_name": self.clear_checkbox_name(name),
            "clearable": self.clearable,
            "preview_imagetype": self.preview_imagetype,
            "preview_fallback_options": self.preview_fallback_options,
            "preview_css_styles": ";".join(self.get_preview_css_styles()),
            "widget": {"type": "file", "name": name, "value": value, "attrs": attrs},
        }

    def render(self, name, value, attrs=None, renderer=None):
        attrs = attrs or {}
        attrs["cradmin-legacy-image-preview-filefield"] = ""
        if self.accept:
            attrs["accept"] = self.accept
        attrs["cradmin-filefield-value"] = value or ""
        imagepath = getattr(value, "name", None)
        imageurl = self.build_preview_url(imagepath)
        context_data = self.get_context_data(imageurl=imageurl, name=name, value=value, attrs=attrs)
        output = render_to_string(self.template_name, context_data, request=self.request)
        return mark_safe(output)


class FileWidget(forms.ClearableFileInput):
    """
    Clearable file widget.

    Can be made un-clearable (for required fields) using the
    ``clearable`` argument for ``__init__``.
    """

    template_name = "cradmin_legacy/filewidgets/filewidget.django.html"

    #: Comma separated string of filetypes that we should accept.
    #: Added to the file upload field as the accept attribute.
    #: Defaults to ``None``, which means that all filetypes are allowed.
    #: Example: ``"application/pdf,application/msword,text/plain"``.
    accept = None

    def __init__(self, attrs=None, template_name=None, clearable=True):
        self.clearable = clearable
        if template_name:
            self.template_name = template_name
        super().__init__(attrs)

    def render(self, name, value, attrs=None, renderer=None):
        attrs = attrs or {}
        if self.accept:
            attrs["accept"] = self.accept
        attrs["cradmin-filefield-value"] = value or ""
        # input_html = forms.FileInput.render(self, name, value, attrs)
        file_path = getattr(value, "name", None)
        if file_path:
            file_name = posixpath.basename(file_path)
        else:
            file_name = None

        output = render_to_string(
            self.template_name,
            {
                # 'input_html': input_html,
                "file_path": file_path,
                "file_name": file_name,
                "MEDIA_URL": settings.MEDIA_URL,
                "clear_checkbox_name": self.clear_checkbox_name(name),
                "clearable": self.clearable,
                "widget": {"type": "file", "name": name, "value": value, "attrs": attrs},
            },
        )
        return mark_safe(output)



import htmls
from unittest import mock
from django import test
from django import forms

from cradmin_legacy.viewhelpers.multiselect2 import target_renderer


class TestTarget(test.TestCase):
    def test_dom_id_default(self):
        selector = htmls.S(target_renderer.Target(form=forms.Form()).render(request=mock.MagicMock()))
        self.assertEqual("cradmin_legacy_multiselect2_select_target", selector.one("form")["id"])

    def test_dom_id_custom(self):
        selector = htmls.S(
            target_renderer.Target(dom_id="customid", form=forms.Form()).render(request=mock.MagicMock())
        )
        self.assertEqual("customid", selector.one("form")["id"])

    def test_form_action_default(self):
        mockrequest = mock.MagicMock()
        mockrequest.get_full_path.return_value = "/form/action"
        selector = htmls.S(target_renderer.Target(form=forms.Form()).render(request=mockrequest))
        self.assertEqual("/form/action", selector.one("form")["action"])

    def test_form_action_custom_parameter(self):
        selector = htmls.S(
            target_renderer.Target(form_action="/some/view", form=forms.Form()).render(request=mock.MagicMock())
        )
        self.assertEqual("/some/view", selector.one("form")["action"])

    def test_form_action_override(self):
        class MyTarget(target_renderer.Target):
            def get_form_action(self, request):
                return "/custom"

        selector = htmls.S(MyTarget(form=forms.Form()).render(request=mock.MagicMock()))
        self.assertEqual("/custom", selector.one("form")["action"])

    def test_without_items_text_default(self):
        selector = htmls.S(target_renderer.Target(form=forms.Form()).render(request=mock.MagicMock()))
        self.assertEqual(
            "", selector.one(".cradmin-legacy-multiselect2-target-without-items-content").alltext_normalized
        )

    def test_without_items_text_custom_parameter(self):
        selector = htmls.S(
            target_renderer.Target(without_items_text="Please make a selection", form=forms.Form()).render(
                request=mock.MagicMock()
            )
        )
        self.assertEqual(
            "Please make a selection",
            selector.one(".cradmin-legacy-multiselect2-target-without-items-content").alltext_normalized,
        )

    def test_without_items_text_override(self):
        class MyTarget(target_renderer.Target):
            def get_without_items_text(self):
                return "No items selected"

        selector = htmls.S(MyTarget(form=forms.Form()).render(request=mock.MagicMock()))
        self.assertEqual(
            "No items selected",
            selector.one(".cradmin-legacy-multiselect2-target-without-items-content").alltext_normalized,
        )

    def test_with_items_title_default(self):
        selector = htmls.S(target_renderer.Target(form=forms.Form()).render(request=mock.MagicMock()))
        self.assertEqual("Selected items", selector.one(".cradmin-legacy-multiselect2-target-title").alltext_normalized)

    def test_with_items_title_custom_parameter(self):
        selector = htmls.S(
            target_renderer.Target(with_items_title="Test title", form=forms.Form()).render(request=mock.MagicMock())
        )
        self.assertEqual("Test title", selector.one(".cradmin-legacy-multiselect2-target-title").alltext_normalized)

    def test_with_items_title_override(self):
        class MyTarget(target_renderer.Target):
            def get_with_items_title(self):
                return "Test title"

        selector = htmls.S(MyTarget(form=forms.Form()).render(request=mock.MagicMock()))
        self.assertEqual("Test title", selector.one(".cradmin-legacy-multiselect2-target-title").alltext_normalized)

    def test_submit_button_text_default(self):
        selector = htmls.S(target_renderer.Target(form=forms.Form()).render(request=mock.MagicMock()))
        self.assertEqual("Submit selection", selector.one('button[type="submit"]').alltext_normalized)

    def test_submit_button_text_custom_parameter(self):
        selector = htmls.S(
            target_renderer.Target(submit_button_text="Perform", form=forms.Form()).render(request=mock.MagicMock())
        )
        self.assertEqual("Perform", selector.one('button[type="submit"]').alltext_normalized)

    def test_submit_button_text_override(self):
        class MyTarget(target_renderer.Target):
            def get_submit_button_text(self):
                return "Test text"

        selector = htmls.S(MyTarget(form=forms.Form()).render(request=mock.MagicMock()))
        self.assertEqual("Test text", selector.one('button[type="submit"]').alltext_normalized)


class TestManyToManySelectTarget(test.TestCase):
    def test_button_angularjs_directive(self):
        selector = htmls.S(
            target_renderer.ManyToManySelectTarget(target_formfield_id="testid", form=forms.Form()).render(
                request=mock.MagicMock()
            )
        )
        self.assertEqual('{"fieldid": "testid"}', selector.one("button")["cradmin-legacy-multiselect2-use-this"])



from datetime import timedelta, datetime
from django.contrib import messages

from django.contrib.auth import get_user_model
from django.urls import reverse
from django.test import TestCase, RequestFactory
from django.utils import timezone
import htmls
from cradmin_legacy.python2_compatibility import mock
from cradmin_legacy.apps.cradmin_activate_account.views.activate import ActivateAccountView

from django_cradmin.apps.cradmin_generic_token_with_metadata.models import GenericTokenWithMetadata
from cradmin_legacy.tests.helpers import create_user


class TestActivateAccountView(TestCase):
    def setUp(self):
        self.testuser = create_user("testuser", is_active=False)

    def __get_url(self, token):
        return reverse("cradmin-activate-account-activate", kwargs={"token": token})

    def _create_generic_token_with_metadata(
        self, user, created_datetime=None, expiration_datetime=None, metadata=None, **kwargs
    ):
        generic_token_with_metadata = GenericTokenWithMetadata(
            created_datetime=(created_datetime or timezone.now()),
            expiration_datetime=(expiration_datetime or (timezone.now() + timedelta(days=2))),
            app="cradmin_activate_account",
            content_object=user,
            **kwargs,
        )
        if metadata:
            generic_token_with_metadata.metadata = metadata
        generic_token_with_metadata.save()
        return generic_token_with_metadata

    def test_get_expired_token(self):
        self._create_generic_token_with_metadata(
            token="valid-token", user=self.testuser, expiration_datetime=datetime(2014, 1, 1)
        )
        response = self.client.get(self.__get_url("valid-token"))
        selector = htmls.S(response.content)
        self.assertTrue(selector.exists("#cradmin_legacy_activate_account_expired_message"))
        self.assertEqual(
            selector.one("#cradmin_legacy_activate_account_expired_message").alltext_normalized,
            "This account activation link has expired.",
        )

    def test_get_invalid_token(self):
        self._create_generic_token_with_metadata(token="valid-token", user=self.testuser)
        response = self.client.get(self.__get_url("invalid-token"))
        selector = htmls.S(response.content)
        self.assertTrue(selector.exists("#cradmin_legacy_activate_account_invalid_token_message"))
        self.assertEqual(
            selector.one("#cradmin_legacy_activate_account_invalid_token_message").alltext_normalized,
            "Invalid account activation URL. Are you sure you copied the entire URL from the email?",
        )

    def test_get_redirect_ok(self):
        self._create_generic_token_with_metadata(
            token="valid-token", user=self.testuser, metadata={"next_url": "/next"}
        )
        response = self.client.get(self.__get_url("valid-token"))
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response["location"], "/next")

    def test_get_activate_ok(self):
        self._create_generic_token_with_metadata(
            token="valid-token", user=self.testuser, metadata={"next_url": "/next"}
        )
        self.assertFalse(self.testuser.is_active)
        self.client.get(self.__get_url("valid-token"))
        testuser = get_user_model().objects.get(id=self.testuser.id)
        self.assertTrue(testuser.is_active)

    def test_post_success_message(self):
        self._create_generic_token_with_metadata(
            token="valid-token", user=self.testuser, metadata={"next_url": "/next"}
        )
        request = RequestFactory().get("/test")
        request.user = self.testuser
        request._messages = mock.MagicMock()
        ActivateAccountView.as_view()(request, token="valid-token")
        request._messages.add.assert_called_once_with(messages.SUCCESS, "Your account is now active.", "")



import logging

from django import http
from django.contrib import messages
from django.core.files.base import ContentFile
from django.utils.translation import gettext_lazy
from crispy_forms import layout
from django import forms
from django.views.generic.edit import FormMixin

from cradmin_legacy.apps.cradmin_temporaryfileuploadstore.models import TemporaryFileCollection
from cradmin_legacy.apps.cradmin_temporaryfileuploadstore.widgets import BulkFileUploadWidget, SingleFileUploadWidget
from cradmin_legacy.utils import crhumanize
from cradmin_legacy.viewhelpers import objecttable
from cradmin_legacy.viewhelpers import crudbase
from cradmin_legacy.viewhelpers import update
from cradmin_legacy.viewhelpers import create
from cradmin_legacy.viewhelpers import delete
from cradmin_legacy.viewhelpers import formbase
from cradmin_legacy import crapp
from cradmin_legacy import crsettings
from cradmin_legacy.apps.cradmin_imagearchive.models import ArchiveImage

logger = logging.getLogger(__name__)


class DescriptionColumn(objecttable.MultiActionColumn):
    modelfield = "description"

    def render_value(self, obj):
        return obj.screenreader_text

    def get_buttons(self, obj):
        edit_description_label = gettext_lazy("Set a description")
        if obj.description:
            edit_description_label = gettext_lazy("Edit description")
        return [
            objecttable.Button(label=edit_description_label, url=self.reverse_appurl("edit", args=[obj.id])),
            objecttable.Button(
                label=gettext_lazy("Delete"),
                url=self.reverse_appurl("delete", args=[obj.id]),
                buttonclass="btn btn-danger btn-sm",
            ),
        ]


class DescriptionSelectColumn(objecttable.UseThisActionColumn):
    modelfield = "description"

    def render_value(self, obj):
        return obj.screenreader_text

    def get_buttons(self, obj):
        return [objecttable.UseThisButton(view=self.view, label=gettext_lazy("Use this"), obj=obj)]


class ImageColumn(objecttable.ImagePreviewColumn):
    modelfield = "image"
    column_width = None

    preview_fallbackoptions = {
        "width": 100,
        "height": 65,
    }

    def get_column_width(self):
        if self.column_width:
            return self.column_width
        else:
            width = crsettings.get_setting("CRADMIN_LEGACY_IMAGEARCHIVE_LISTING_IMAGEWIDTH", 100)
            return f"{width}px"

    def get_preview_imagetype(self):
        imagetype_from_settings = crsettings.get_setting("CRADMIN_LEGACY_IMAGEARCHIVE_LISTING_IMAGETYPE")
        if self.preview_imagetype:
            return self.preview_imagetype
        elif imagetype_from_settings:
            return imagetype_from_settings
        else:
            return None

    def get_header(self):
        return gettext_lazy("Preview")


class ArchiveImagesQuerySetForRoleMixin:
    """
    Used by listing, update and delete view to ensure
    that only images that the current role has access to
    is available.
    """

    def get_queryset_for_role(self, role):
        return ArchiveImage.objects.filter_owned_by_role(role).order_by("-created_datetime")


def get_bulkupload_apiparameters():
    apiparameters = {
        "accept": "image/png,image/jpeg,image/gif",
    }
    max_filesize_bytes = crsettings.get_setting("CRADMIN_LEGACY_IMAGEARCHIVE_MAX_FILESIZE", None)
    if max_filesize_bytes is not None:
        apiparameters["max_filesize_bytes"] = crhumanize.dehumanize_readable_filesize(max_filesize_bytes)
    return apiparameters


class BulkAddForm(forms.Form):
    filecollectionid = forms.IntegerField(
        required=True,
        widget=BulkFileUploadWidget(
            autosubmit=True,
            accept="image/*",
            # accept='image/png,image/jpeg,image/gif',  # NOTE: Does not work with the fileselector in firefox
            apiparameters=get_bulkupload_apiparameters(),
            dropbox_text=gettext_lazy("Upload images by dragging and dropping them here"),
            invalid_filetype_message=gettext_lazy("Invalid filetype. You can only upload images."),
            advanced_fileselectbutton_text=gettext_lazy("... or select images"),
            simple_fileselectbutton_text=gettext_lazy("Select images ..."),
        ),
        label=gettext_lazy("Upload at least one image"),
        error_messages={"required": gettext_lazy("You must upload at least one image.")},
    )


class SingleAddForm(forms.Form):
    filecollectionid = forms.IntegerField(
        required=True,
        widget=SingleFileUploadWidget(
            autosubmit=True,
            accept="image/*",
            apiparameters=get_bulkupload_apiparameters(),
            dropbox_text=gettext_lazy("Upload an image by dragging and dropping it here"),
            advanced_fileselectbutton_text=gettext_lazy("... or select an image"),
            invalid_filetype_message=gettext_lazy("Invalid filetype. You can only upload images."),
            simple_fileselectbutton_text=gettext_lazy("Select an image ..."),
        ),
        label=gettext_lazy("Upload an image"),
        error_messages={"required": gettext_lazy("You must upload an image.")},
    )

    def clean(self):
        cleaned_data = super().clean()
        filecollectionid = cleaned_data.get("filecollectionid", None)
        if filecollectionid is not None:
            collection = TemporaryFileCollection.objects.get(id=filecollectionid)
            filecount = collection.files.count()
            if filecount < 1:
                raise forms.ValidationError({"filecollectionid": gettext_lazy("You must upload an image.")})
            elif filecount > 1:
                raise forms.ValidationError({"filecollectionid": gettext_lazy("You must upload exactly one image.")})


class AddImageOverlayButton(objecttable.NonSubmitButton):
    def get_html_attributes(self):
        attributes = super().get_html_attributes()
        attributes["cradmin-legacy-bulkfileupload-show-overlay"] = "filecollectionid"
        return attributes


class BaseImagesListView(
    ArchiveImagesQuerySetForRoleMixin, objecttable.ObjectTableView, formbase.FormViewMixin, FormMixin
):
    searchfields = ["name", "description", "file_extension"]
    hide_column_headers = True
    model = ArchiveImage
    paginate_by = 15

    form_id = "cradmin_legacy_imagearchive_bulkadd_form"
    extra_form_css_classes = ["cradmin-legacy-form-noasterisk", "cradmin-legacy-bulkfileupload-form-overlay"]
    template_name = "cradmin_legacy/apps/cradmin_imagearchive/listview.django.html"

    def get_form_attributes(self):
        return {
            "cradmin-legacy-bulkfileupload-form": "",
            "cradmin-legacy-bulkfileupload-form-overlay": "true",
            "cradmin-legacy-bulkfileupload-form-open-overlay-on-window-dragdrop": "true",
        }

    def get_field_layout(self):
        return [
            layout.Div(
                "filecollectionid",
                # css_class="cradmin-globalfields"),
                css_class="cradmin-focusfield",
            ),
        ]

    def get_buttons(self):
        return [AddImageOverlayButton(label=gettext_lazy("Add image"), buttonclass="btn btn-primary")]

    def get_button_layout(self):
        # Overridden because get_buttons from formbase.FormView and
        # objecttable.ObjectTableView both use get_buttons().
        return []

    def get_formhelper(self):
        formhelper = super().get_formhelper()
        formhelper.form_show_labels = False
        return formhelper

    def upload_file_to_archive(self, temporaryfile):
        archiveimage = ArchiveImage(role=self.request.cradmin_role, name=temporaryfile.filename)
        archiveimage.file_size = temporaryfile.file.size
        archiveimage.clean()
        archiveimage.save()
        archiveimage.image.save(temporaryfile.filename, ContentFile(temporaryfile.file.read()))
        archiveimage.full_clean()
        return archiveimage

    def upload_files_to_archive(self, temporaryfilecollection):
        uploaded_archiveimages = []
        for temporaryfile in temporaryfilecollection.files.all():
            uploaded_archiveimage = self.upload_file_to_archive(temporaryfile)
            uploaded_archiveimages.append(uploaded_archiveimage)
        return uploaded_archiveimages

    def get_collectionqueryset(self):
        return TemporaryFileCollection.objects.filter_for_user(self.request.user).prefetch_related("files")

    def get_success_message(self, temporaryfilecollection):
        """
        Override this to provide a success message.

        The ``temporaryfilecollection`` is the TemporaryFileCollection that was just uploaded.

        Used by :meth:`.add_success_messages`.
        """
        filenames = [f'"{temporaryfile.filename}"' for temporaryfile in temporaryfilecollection.files.all()]
        return gettext_lazy("Uploaded %(what)s.") % {"what": ",".join(filenames)}

    def add_success_messages(self, temporaryfilecollection):
        """
        Add success messages on successful upload.

        The ``temporaryfilecollection`` is the TemporaryFileCollection that was just uploaded.

        Defaults to add :meth:`.get_success_message` as a django messages
        success message if :meth:`.get_success_message` returns anything.

        You can override this to add multiple messages or to show messages in some other way.
        """
        success_message = self.get_success_message(temporaryfilecollection)
        if success_message:
            messages.success(self.request, success_message)

    def form_valid(self, form):
        collectionid = form.cleaned_data["filecollectionid"]
        try:
            temporaryfilecollection = self.get_collectionqueryset().get(id=collectionid)
        except TemporaryFileCollection.DoesNotExist:
            return http.HttpResponseNotFound()
        else:
            # We use ``uploaded_archiveimages`` in :meth:`.ArchiveImagesSingleSelectView.get_success_url`
            self.uploaded_archiveimages = self.upload_files_to_archive(temporaryfilecollection)
            self.add_success_messages(temporaryfilecollection)
            temporaryfilecollection.clear_files_and_delete()
            return http.HttpResponseRedirect(self.get_success_url())

    def form_invalid(self, form):
        self.object_list = self.get_queryset()
        return super().form_invalid(form)

    def post(self, request, *args, **kwargs):
        form_class = self.get_form_class()
        form = self.get_form(form_class)
        if form.is_valid():
            return self.form_valid(form)
        else:
            return self.form_invalid(form)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        self.add_context_data(context)
        if "form" not in context:
            form_class = self.get_form_class()
            form = self.get_form(form_class)
            context["form"] = form
        return context


class ArchiveImagesListView(BaseImagesListView):
    """
    The list view in the image achive app.
    """

    columns = [
        ImageColumn,
        DescriptionColumn,
    ]
    form_class = BulkAddForm

    # def get_button_layout(self):
    #     return [
    #         layout.Div(PrimarySubmit('save', gettext_lazy('Upload images')),
    #                    css_class="cradmin_legacy_submitrow")
    #     ]


class ArchiveImagesSingleSelectView(BaseImagesListView):
    """
    Used when selecting a single archive image as a foreign key.
    """

    columns = [
        ImageColumn,
        DescriptionSelectColumn,
    ]
    hide_menu = True
    form_class = SingleAddForm
    listing_viewname = "singleselect"

    def make_foreignkey_preview_for(self, obj):
        archiveimage = obj
        return archiveimage.get_preview_html(request=self.request)

    # def get_button_layout(self):
    #     return [
    #         layout.Div(PrimarySubmit('save', gettext_lazy('Upload image')),
    #                    css_class="cradmin_legacy_submitrow")
    #     ]

    def get_success_url(self):
        url = self.request.build_absolute_uri()
        uploaded_archiveimage = self.uploaded_archiveimages[0]
        url = create.CreateView.add_foreignkey_selected_value_to_url_querystring(url, uploaded_archiveimage.pk)
        return str(url)


class ArchiveImageUpdateView(crudbase.OnlySaveButtonMixin, ArchiveImagesQuerySetForRoleMixin, update.UpdateView):
    """
    View used to create edit existing images.
    """

    model = ArchiveImage
    fields = ["description"]
    roleid_field = "role"

    def get_field_layout(self):
        return [
            layout.Div(
                layout.Field("description", css_class="cradmin-textarea-small"), css_class="cradmin-globalfields"
            )
        ]


class ArchiveImageDeleteView(ArchiveImagesQuerySetForRoleMixin, delete.DeleteView):
    """
    View used to delete existing images.
    """


class App(crapp.App):
    appurls = [
        crapp.Url(r"^$", ArchiveImagesListView.as_view(), name=crapp.INDEXVIEW_NAME),
        crapp.Url(r"^singleselect$", ArchiveImagesSingleSelectView.as_view(), name="singleselect"),
        crapp.Url(r"^edit/(?P<pk>\d+)$", ArchiveImageUpdateView.as_view(), name="edit"),
        crapp.Url(r"^delete/(?P<pk>\d+)$", ArchiveImageDeleteView.as_view(), name="delete"),
    ]



import logging

from django.test import TestCase
from model_bakery import baker

from cradmin_legacy.python2_compatibility import mock
from cradmin_legacy.tests.viewhelpers.cradmin_viewhelpers_testapp.models import (
    FilterTestModel,
)
from cradmin_legacy.viewhelpers.listfilter.base.abstractfilter import AbstractFilter
from cradmin_legacy.viewhelpers.listfilter.base.exceptions import (
    InvalidFiltersStringError,
)
from cradmin_legacy.viewhelpers.listfilter.base.filtershandler import (
    FiltersHandler,
    logger,
)


class MinimalIntFilter(AbstractFilter):
    def get_slug(self):
        return "i"

    def clean_value(self, value):
        return int(value)


class MinimalStringFilter(AbstractFilter):
    def get_slug(self):
        return "s"


class TestFiltersHandler(TestCase):
    def test_split_raw_filter_values(self):
        filtershandler = FiltersHandler(urlbuilder=mock.MagicMock())
        self.assertEqual(["a/b", "c,d"], filtershandler.split_raw_filter_values("a%252Fb%2Cc%252Cd"))

    def test_join_filter_values(self):
        filtershandler = FiltersHandler(urlbuilder=mock.MagicMock())
        self.assertEqual("a%252Fb%2Cc%252Cd", filtershandler.join_filter_values(["a/b", "c,d"]))

    def test_invalid_filter_string(self):
        filtershandler = FiltersHandler(urlbuilder=mock.MagicMock())
        with self.assertRaisesMessage(InvalidFiltersStringError, '"x" does not contain "-".'):
            filtershandler.parse("x")

    def test_invalid_filter_slug(self):
        filtershandler = FiltersHandler(urlbuilder=mock.MagicMock())
        with self.assertLogs(logger, level=logging.DEBUG) as logcontext:
            filtershandler.parse("x-10")
            self.assertEqual(len(logcontext.records), 1)
            self.assertEqual(logcontext.records[0].message, '"x" is not a valid filter slug.')

    def test_add_filter_duplicate_filter_slug(self):
        filtershandler = FiltersHandler(urlbuilder=mock.MagicMock())
        filtershandler.add_filter(AbstractFilter(slug="x"))
        with self.assertRaisesMessage(ValueError, 'Duplicate slug: "x".'):
            filtershandler.add_filter(AbstractFilter(slug="x"))

    def test_add_filter_invalid_filter_slug(self):
        filtershandler = FiltersHandler(urlbuilder=mock.MagicMock())
        with self.assertRaisesMessage(ValueError, 'Invalid filter slug: "x-y". Slugs can not contain "-".'):
            filtershandler.add_filter(AbstractFilter(slug="x-y"))

    def test_parse_empty_string(self):
        filtershandler = FiltersHandler(urlbuilder=mock.MagicMock())
        filtershandler.parse("")
        self.assertEqual({}, filtershandler.filtermap)

    def test_parse_none(self):
        filtershandler = FiltersHandler(urlbuilder=mock.MagicMock())
        filtershandler.parse(None)
        self.assertEqual({}, filtershandler.filtermap)

    def test_parse_simple_valid_filter_string(self):
        filtershandler = FiltersHandler(urlbuilder=mock.MagicMock())
        filtershandler.add_filter(MinimalStringFilter())
        filtershandler.parse("s-test")
        with self.assertNoLogs(logger, level=logging.DEBUG):
            self.assertEqual(["test"], filtershandler.filtermap["s"].values)

    def test_parse_multivalue_valid_filter_string(self):
        filtershandler = FiltersHandler(urlbuilder=mock.MagicMock())
        filtershandler.add_filter(MinimalStringFilter())
        filtershandler.parse("s-a,b,c")
        self.assertEqual(["a", "b", "c"], filtershandler.filtermap["s"].values)

    def test_parse_simple_valid_filter_string_trailing_slash(self):
        filtershandler = FiltersHandler(urlbuilder=mock.MagicMock())
        filtershandler.add_filter(MinimalStringFilter())
        filtershandler.parse("s-test/")
        self.assertEqual(["test"], filtershandler.filtermap["s"].values)

    def test_parse_simple_valid_filter_string_leading_slash(self):
        filtershandler = FiltersHandler(urlbuilder=mock.MagicMock())
        filtershandler.add_filter(MinimalStringFilter())
        filtershandler.parse("/s-test")
        self.assertEqual(["test"], filtershandler.filtermap["s"].values)

    def test_parse_complex_valid_filter_string(self):
        filtershandler = FiltersHandler(urlbuilder=mock.MagicMock())
        filtershandler.add_filter(MinimalStringFilter())
        filtershandler.add_filter(MinimalIntFilter())
        filtershandler.parse("/i-10/s-jane,joe/")
        self.assertEqual(2, len(filtershandler.filtermap))
        self.assertEqual(["10"], filtershandler.filtermap["i"].values)
        self.assertEqual(["jane", "joe"], filtershandler.filtermap["s"].values)

    def test_build_filters_string(self):
        filtershandler = FiltersHandler(urlbuilder=mock.MagicMock())
        intfilter = MinimalIntFilter()
        intfilter.set_values(values=["10"])
        stringfilter = MinimalStringFilter()
        stringfilter.set_values(values=["jane", "joe"])
        filtershandler.add_filter(intfilter)
        filtershandler.add_filter(stringfilter)

        new_stringfilter = stringfilter.copy()
        new_stringfilter.set_values(values=["jack", "peter"])
        self.assertEqual(
            "i-10/s-jack%2Cpeter", filtershandler.build_filters_string(changed_filterobject=new_stringfilter)
        )

    def test_build_filter_url(self):
        def urlbuilder(filters_string):
            return f"/the/prefix/{filters_string}?a=querystring"

        filtershandler = FiltersHandler(urlbuilder=urlbuilder)
        intfilter = MinimalIntFilter()
        intfilter.set_values(values=["10"])
        stringfilter = MinimalStringFilter()
        stringfilter.set_values(values=["jane", "joe"])
        filtershandler.add_filter(intfilter)
        filtershandler.add_filter(stringfilter)

        new_stringfilter = stringfilter.copy()
        new_stringfilter.set_values(values=["jack", "peter"])
        self.assertEqual(
            "/the/prefix/i-10/s-jack%2Cpeter?a=querystring",
            filtershandler.build_filter_url(changed_filterobject=new_stringfilter),
        )

    def test_filter(self):
        class FilterOne(AbstractFilter):
            def filter(self, queryobject):
                return queryobject.filter(mycharfield="test")

        class FilterTwo(AbstractFilter):
            def filter(self, queryobject):
                return queryobject.filter(mybooleanfield=True)

        match = baker.make("cradmin_viewhelpers_testapp.FilterTestModel", mycharfield="test", mybooleanfield=True)
        baker.make("cradmin_viewhelpers_testapp.FilterTestModel", mycharfield="no match", mybooleanfield=True)
        baker.make("cradmin_viewhelpers_testapp.FilterTestModel", mycharfield="test", mybooleanfield=False)

        filtershandler = FiltersHandler(urlbuilder=mock.MagicMock())
        filtershandler.add_filter(FilterOne(slug="filterone"))
        filtershandler.add_filter(FilterTwo(slug="filtertwo"))
        queryset = filtershandler.filter(FilterTestModel.objects.all())
        self.assertEqual({match}, set(queryset))

    def test_filter_exclude(self):
        class FilterOne(AbstractFilter):
            def filter(self, queryobject):
                return queryobject.filter(mycharfield="test")

        class FilterTwo(AbstractFilter):
            def filter(self, queryobject):
                return queryobject.filter(mybooleanfield=True)

        match1 = baker.make("cradmin_viewhelpers_testapp.FilterTestModel", mycharfield="test", mybooleanfield=True)
        match2 = baker.make("cradmin_viewhelpers_testapp.FilterTestModel", mycharfield="no match", mybooleanfield=True)
        baker.make("cradmin_viewhelpers_testapp.FilterTestModel", mycharfield="test", mybooleanfield=False)

        filtershandler = FiltersHandler(urlbuilder=mock.MagicMock())
        filtershandler.add_filter(FilterOne(slug="filterone"))
        filtershandler.add_filter(FilterTwo(slug="filtertwo"))
        queryset = filtershandler.filter(FilterTestModel.objects.all(), exclude={"filterone"})
        self.assertEqual({match1, match2}, set(queryset))

    def test_get_label_for(self):
        class MyFilter(AbstractFilter):
            pass

        filtershandler = FiltersHandler(urlbuilder=mock.MagicMock())
        filtershandler.add_filter(MyFilter(label="Test Label", slug="test"))
        self.assertEqual("Test Label", filtershandler.get_label_for("test"))

    def test_get_cleaned_value_for(self):
        filtershandler = FiltersHandler(urlbuilder=mock.MagicMock())
        filtershandler.add_filter(MinimalIntFilter())
        filtershandler.parse("/i-10")
        self.assertEqual(10, filtershandler.get_cleaned_value_for("i"))

    def test_get_cleaned_values_for(self):
        filtershandler = FiltersHandler(urlbuilder=mock.MagicMock())
        filtershandler.add_filter(MinimalIntFilter())
        filtershandler.parse("/i-10")
        self.assertEqual([10], filtershandler.get_cleaned_values_for("i"))



from django.views.generic import TemplateView

from cradmin_legacy.apps.cradmin_resetpassword.views.begin import PasswordResetEmail


class EmailSentView(TemplateView):
    template_name = "cradmin_resetpassword/email_sent.django.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["email_subject"] = PasswordResetEmail().render_subject()
        return context



from cradmin_legacy.viewhelpers.listfilter.base.abstractfilter import AbstractFilter


class AbstractCheckboxFilter(AbstractFilter):
    """
    Abstract base class for any checkbox based multiselect filter.

    You only have to override :meth:`~AbstractCheckboxFilter.get_choices` and
    :meth:`~AbstractCheckboxFilter.filter`.
    """

    template_name = "cradmin_legacy/viewhelpers/listfilter/django/multi/checkbox/base.django.html"

    def get_choices(self):
        """
        Get choices as a list of of ``(value, label)`` pairs.
        """
        raise NotImplementedError()

    def get_choices_cached(self):
        if not hasattr(self, "_choices"):
            self._choices = self.get_choices()
        return self._choices

    def get_choicesdata(self):
        """
        Get choices data as a list of dicts.

        This uses :meth:`.get_choices`, and turns it into
        a list of dicts where each dict has:

        - ``url``: The URL to load when making a choice.
          This is generated using
          :meth:`~cradmin_legacy.viewhelpers.listfilter.base.AbstractFilter.build_set_values_url`
          with the value from :meth:`.get_choices` as input.
        - ``label``: The label to show (taken directly from :meth:`.get_choices`.
        - ``is_selected`` (boolean): If the current value matches the value of a choice,
          that choice will be selected. If none of them matches, the first choice is selected.
          You can change this behavior by overriding :meth:`.get_default_is_selected_index`.
        - ``dom_id``: The DOM ID of the checkbox input field.

        Returns:
            A tuple with the list of dicts described above and the selected label.
        """
        selected_values = set(self.get_cleaned_values())
        choicesdata = []
        for value, label in self.get_choices_cached():
            is_selected = value in selected_values
            if is_selected:
                url = self.build_remove_values_url(values=[value])
            else:
                url = self.build_add_values_url(values=[value])
            choicesdata.append(
                {
                    "url": url,
                    "label": label,
                    "is_selected": is_selected,
                    "dom_id": f"{self.get_inputfield_dom_id()}_{value}",
                }
            )
        return choicesdata

    def get_context_data(self, request=None):
        context = super().get_context_data(request=None)
        context["choicesdata"] = self.get_choicesdata()
        return context

    def get_cleaned_values(self):
        cleaned_values = super().get_cleaned_values()
        choices = {value for value, label in self.get_choices_cached()}
        return [value for value in cleaned_values if value in choices]



from django.contrib import admin
from cradmin_legacy.apps.cradmin_temporaryfileuploadstore.models import TemporaryFileCollection, TemporaryFile


class TemporaryFileInline(admin.StackedInline):
    fields = ["filename", "file"]
    readonly_fields = ["filename", "file"]
    model = TemporaryFile
    extra = 0


class TemporaryFileCollectionAdmin(admin.ModelAdmin):
    inlines = [TemporaryFileInline]
    list_display = (
        "id",
        "user",
        "created_datetime",
        "minutes_to_live",
    )
    readonly_fields = ["singlemode", "user", "accept", "created_datetime", "unique_filenames", "max_filename_length"]
    search_fields = [
        "id",
        "user__id",
        "user__email",
    ]
    date_hierarchy = "created_datetime"

    def get_queryset(self, request):
        if request.user.is_superuser:
            return super().get_queryset(request)
        else:
            return TemporaryFile.objects.none()


admin.site.register(TemporaryFileCollection, TemporaryFileCollectionAdmin)



"""
An example app using Sortable.
"""

from django.db import models

from cradmin_legacy.sortable.models import SortableBase, SortableManagerBase


class ItemContainer(models.Model):
    """
    A test class for being a container for items that should be sorted.

    It has a name field only for testing purposes.
    """

    name = models.CharField(max_length=255, blank=True, null=False, default="")

    def __str__(self):
        return f"Item container {self.id}, {self.name}"


class SortableItemManager(SortableManagerBase):
    """
    Sortable items that inherit SortableBase must also have a manager that inherits SortableManagerBase.

    The `parent_attribute` must be set, and it must have the name of the parent in which the items belong.
    """

    parent_attribute = "container"


class SortableItem(SortableBase):
    """
    The sortable item. The important thing here is the usage og SortableItemManager.

    The `sort_index` field is inherited from SortableBase.
    """

    container = models.ForeignKey(ItemContainer, blank=False, null=False, on_delete=models.CASCADE)
    name = models.CharField(max_length=255, blank=True, null=False, default="")
    objects = SortableItemManager()

    def __str__(self):
        return f"Id: {self.id}, Sort index: {self.sort_index}, Name: {self.name}, {self.container}"
